@@ -2,10 +2,12 @@
 
 import os
 import sys
+
 import yaml
 
+
 # Need to import modules that lie on an upward-relative path
-sys.path.append(os.path.join(sys.path[0], '..'))
+sys.path.append(os.path.dirname(sys.path[0]))
 
 import cimodel.lib.miniyaml as miniyaml
 
