@@ -24,7 +24,11 @@ static void imageYUV420CenterCropToFloatBuffer(
     jfloatArray jnormMeanRGB,
     jfloatArray jnormStdRGB,
     jobject outBuffer,
-    jint outOffset) {
+    jint outOffset,
+    jint memoryFormatCode) {
+  constexpr static int32_t kMemoryFormatContiguous = 1;
+  constexpr static int32_t kMemoryFormatChannelsLast = 2;
+
   float* outData = (float*)jniEnv->GetDirectBufferAddress(outBuffer);
 
   jfloat normMeanRGB[3];
@@ -91,32 +95,66 @@ static void imageYUV420CenterCropToFloatBuffer(
   int xBeforeRtn, yBeforeRtn;
   int yi, yIdx, uvIdx, ui, vi, a0, ri, gi, bi;
   int channelSize = tensorWidth * tensorHeight;
-  int wr = outOffset;
-  int wg = wr + channelSize;
-  int wb = wg + channelSize;
-  for (int x = 0; x < tensorWidth; x++) {
+  // A bit of code duplication to avoid branching in the cycles
+  if (memoryFormatCode == kMemoryFormatContiguous) {
+    int wr = outOffset;
+    int wg = wr + channelSize;
+    int wb = wg + channelSize;
     for (int y = 0; y < tensorHeight; y++) {
-      xBeforeRtn = cropXAdd + cropXMult * (int)(x * scale);
-      yBeforeRtn = cropYAdd + cropYMult * (int)(y * scale);
-      yIdx = yBeforeRtn * yRowStride + xBeforeRtn * yPixelStride;
-      uvIdx = (yBeforeRtn >> 1) * uvRowStride + (xBeforeRtn >> 1) * uvPixelStride;
-      ui = uData[uvIdx];
-      vi = vData[uvIdx];
-      yi = yData[yIdx];
-      yi = (yi - 16) < 0 ? 0 : (yi - 16);
-      ui -= 128;
-      vi -= 128;
-      a0 = 1192 * yi;
-      ri = (a0 + 1634 * vi) >> 10;
-      gi = (a0 - 833 * vi - 400 * ui) >> 10;
-      bi = (a0 + 2066 * ui) >> 10;
-      ri = ri > 255 ? 255 : ri < 0 ? 0 : ri;
-      gi = gi > 255 ? 255 : gi < 0 ? 0 : gi;
-      bi = bi > 255 ? 255 : bi < 0 ? 0 : bi;
-      outData[wr++] = (ri - normMeanRm255) / normStdRm255;
-      outData[wg++] = (gi - normMeanGm255) / normStdGm255;
-      outData[wb++] = (bi - normMeanBm255) / normStdBm255;
+      for (int x = 0; x < tensorWidth; x++) {
+        xBeforeRtn = cropXAdd + cropXMult * (int)(x * scale);
+        yBeforeRtn = cropYAdd + cropYMult * (int)(y * scale);
+        yIdx = yBeforeRtn * yRowStride + xBeforeRtn * yPixelStride;
+        uvIdx =
+            (yBeforeRtn >> 1) * uvRowStride + (xBeforeRtn >> 1) * uvPixelStride;
+        ui = uData[uvIdx];
+        vi = vData[uvIdx];
+        yi = yData[yIdx];
+        yi = (yi - 16) < 0 ? 0 : (yi - 16);
+        ui -= 128;
+        vi -= 128;
+        a0 = 1192 * yi;
+        ri = (a0 + 1634 * vi) >> 10;
+        gi = (a0 - 833 * vi - 400 * ui) >> 10;
+        bi = (a0 + 2066 * ui) >> 10;
+        ri = ri > 255 ? 255 : ri < 0 ? 0 : ri;
+        gi = gi > 255 ? 255 : gi < 0 ? 0 : gi;
+        bi = bi > 255 ? 255 : bi < 0 ? 0 : bi;
+        outData[wr++] = (ri - normMeanRm255) / normStdRm255;
+        outData[wg++] = (gi - normMeanGm255) / normStdGm255;
+        outData[wb++] = (bi - normMeanBm255) / normStdBm255;
+      }
     }
+  } else if (memoryFormatCode == kMemoryFormatChannelsLast) {
+    int wc = outOffset;
+    for (int y = 0; y < tensorHeight; y++) {
+      for (int x = 0; x < tensorWidth; x++) {
+        xBeforeRtn = cropXAdd + cropXMult * (int)(x * scale);
+        yBeforeRtn = cropYAdd + cropYMult * (int)(y * scale);
+        yIdx = yBeforeRtn * yRowStride + xBeforeRtn * yPixelStride;
+        uvIdx =
+            (yBeforeRtn >> 1) * uvRowStride + (xBeforeRtn >> 1) * uvPixelStride;
+        ui = uData[uvIdx];
+        vi = vData[uvIdx];
+        yi = yData[yIdx];
+        yi = (yi - 16) < 0 ? 0 : (yi - 16);
+        ui -= 128;
+        vi -= 128;
+        a0 = 1192 * yi;
+        ri = (a0 + 1634 * vi) >> 10;
+        gi = (a0 - 833 * vi - 400 * ui) >> 10;
+        bi = (a0 + 2066 * ui) >> 10;
+        ri = ri > 255 ? 255 : ri < 0 ? 0 : ri;
+        gi = gi > 255 ? 255 : gi < 0 ? 0 : gi;
+        bi = bi > 255 ? 255 : bi < 0 ? 0 : bi;
+        outData[wc++] = (ri - normMeanRm255) / normStdRm255;
+        outData[wc++] = (gi - normMeanGm255) / normStdGm255;
+        outData[wc++] = (bi - normMeanBm255) / normStdBm255;
+      }
+    }
+  } else {
+    jclass Exception = jniEnv->FindClass("java/lang/IllegalArgumentException");
+    jniEnv->ThrowNew(Exception, "Illegal memory format code");
   }
 }
 } // namespace pytorch_vision_jni
@@ -135,7 +173,7 @@ JNIEXPORT jint JNI_OnLoad(JavaVM* vm, void*) {
 
   static const JNINativeMethod methods[] = {
       {"imageYUV420CenterCropToFloatBuffer",
-       "(Ljava/nio/ByteBuffer;IILjava/nio/ByteBuffer;Ljava/nio/ByteBuffer;IIIIIII[F[FLjava/nio/Buffer;I)V",
+       "(Ljava/nio/ByteBuffer;IILjava/nio/ByteBuffer;Ljava/nio/ByteBuffer;IIIIIII[F[FLjava/nio/Buffer;II)V",
        (void*)pytorch_vision_jni::imageYUV420CenterCropToFloatBuffer},
   };
   int rc = env->RegisterNatives(
