@@ -1,16 +1,24 @@
-#include <ATen/ATen.h>
-#include <ATen/LegacyTHFunctionsCPU.h>
+#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
+#include <ATen/core/Tensor.h>
+#include <ATen/Dispatch.h>
 #include <ATen/TensorUtils.h>
-#include <ATen/Utils.h>
-#include <ATen/div_rtn.h>
 
 #include <ATen/native/im2col.h>
 #include <ATen/native/im2col_shape_check.h>
+#include <c10/util/irange.h>
 
-namespace at {
-namespace native {
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/col2im_native.h>
+#include <ATen/ops/empty_like.h>
+#include <ATen/ops/im2col_native.h>
+#endif
+
+namespace at::native {
 namespace {
-  
+
 static void im2col_out_cpu_template(
     Tensor& output,
     const Tensor& input_,
@@ -65,7 +73,7 @@ static void im2col_out_cpu_template(
 
   if (input.dim() == 3) {
     batched_input = false;
-    input.resize_({1, input.size(0), input.size(1), input.size(2)});
+    input = input.view({1, input.size(0), input.size(1), input.size(2)});
   }
 
   int64_t batch_size = input.size(0);
@@ -85,19 +93,18 @@ static void im2col_out_cpu_template(
   int64_t output_length = output_height * output_width;
 
   output.resize_({batch_size, n_output_plane, output_length});
-  output.zero_();
 
-  AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND1(kHalf,
+  AT_DISPATCH_FLOATING_AND_COMPLEX_TYPES_AND3(kBFloat16, kHalf, kBool,
       input.scalar_type(), "im2col_out_cpu", [&] {
         Tensor input_n;
         Tensor output_n;
 
-        for (int64_t elt = 0; elt < batch_size; elt++) {
+        for (const auto elt : c10::irange(batch_size)) {
           input_n = input.select(0, elt);
           output_n = output.select(0, elt);
 
           im2col<scalar_t>(
-              input_n.data_ptr<scalar_t>(),
+              input_n.const_data_ptr<scalar_t>(),
               n_input_plane,
               input_height,
               input_width,
@@ -111,7 +118,7 @@ static void im2col_out_cpu_template(
               stride_width,
               dilation_height,
               dilation_width,
-              output_n.data_ptr<scalar_t>());
+              output_n.mutable_data_ptr<scalar_t>());
         }
 
         if (!batched_input) {
@@ -120,38 +127,14 @@ static void im2col_out_cpu_template(
       });
 }
 
-static void im2col_backward_out_cpu_template(
-    Tensor& grad_input,
-    const Tensor& grad_output,
-    IntArrayRef input_size,
-    IntArrayRef kernel_size,
-    IntArrayRef dilation,
-    IntArrayRef padding,
-    IntArrayRef stride) {
-  TORCH_CHECK(
-      input_size.size() == 2,
-      "It is expected input_size equals to 2, but got size ",
-      input_size.size());
-  // col2im_out_cpu checks size of kernel_size, dilation, padding and stride
-  col2im_out_cpu(
-      grad_input,
-      grad_output,
-      input_size,
-      kernel_size,
-      dilation,
-      padding,
-      stride);
-}
-
 } // namespace
 
-Tensor& im2col_out_cpu(
-    Tensor& output,
-    const Tensor& input,
+Tensor& im2col_out_cpu(const Tensor& input,
     IntArrayRef kernel_size,
     IntArrayRef dilation,
     IntArrayRef padding,
-    IntArrayRef stride) {
+    IntArrayRef stride,
+    Tensor& output) {
   im2col_out_cpu_template(
       output, input, kernel_size, dilation, padding, stride);
   return output;
@@ -170,44 +153,4 @@ Tensor im2col_cpu(
   return output;
 }
 
-Tensor& im2col_backward_out_cpu(
-    Tensor& grad_input,
-    const Tensor& grad_output,
-    IntArrayRef input_size,
-    IntArrayRef kernel_size,
-    IntArrayRef dilation,
-    IntArrayRef padding,
-    IntArrayRef stride) {
-  im2col_backward_out_cpu_template(
-      grad_input,
-      grad_output,
-      input_size,
-      kernel_size,
-      dilation,
-      padding,
-      stride);
-  return grad_input;
-}
-
-Tensor im2col_backward_cpu(
-    const Tensor& grad_output,
-    IntArrayRef input_size,
-    IntArrayRef kernel_size,
-    IntArrayRef dilation,
-    IntArrayRef padding,
-    IntArrayRef stride) {
-  Tensor grad_input = at::empty_like(grad_output, LEGACY_CONTIGUOUS_MEMORY_FORMAT);
-
-  im2col_backward_out_cpu_template(
-      grad_input,
-      grad_output,
-      input_size,
-      kernel_size,
-      dilation,
-      padding,
-      stride);
-  return grad_input;
-}
-
-} // namespace native
-} // namespace at
+} // namespace at::native
