@@ -1,10 +1,20 @@
-#include <ATen/ATen.h>
-#include <ATen/Dispatch.h>
+#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
+#include <ATen/core/Tensor.h>
 #include <ATen/AccumulateType.h>
+#include <ATen/Dispatch.h>
 #include <ATen/native/LossMulti.h>
+#include <c10/util/irange.h>
 
-namespace at {
-namespace native {
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/empty.h>
+#include <ATen/ops/multi_margin_loss_backward_native.h>
+#include <ATen/ops/multi_margin_loss_native.h>
+#endif
+
+namespace at::native {
 
 namespace {
 
@@ -18,7 +28,7 @@ inline scalar_t multi_margin_inner_sum_cpu(
     const int64_t target_idx) {
   const scalar_t input_target = input_data[target_idx];
   scalar_t sum = 0;
-  for (int64_t d = 0; d < dim; d++) {
+  for (const auto d : c10::irange(dim)) {
     if (d == target_idx) {
       continue;
     }
@@ -49,11 +59,11 @@ inline int64_t target_index_checked(
 template <typename scalar_t>
 static inline void multi_margin_loss_cpu_kernel(
     Tensor& output,
-    scalar_t* input_data,
-    int64_t* target_data,
+    const scalar_t* input_data,
+    const int64_t* target_data,
     const int p,
     scalar_t margin,
-    scalar_t* weight_data,
+    const scalar_t* weight_data,
     const int64_t nframe,
     const int64_t dim,
     const int64_t reduction) {
@@ -63,7 +73,7 @@ static inline void multi_margin_loss_cpu_kernel(
   // cannot be handled by TensorAccessor)
   if (reduction == Reduction::None && output.dim() > 0) {
     auto output_acc = output.accessor<scalar_t, 1>();
-    for (int64_t t = 0; t < nframe; t++) {
+    for (const auto t : c10::irange(nframe)) {
       const auto idx = target_index_checked(target_data, t, dim);
       auto sum = multi_margin_inner_sum_cpu(
           input_data, weight_data, p, margin, dim, idx);
@@ -73,7 +83,7 @@ static inline void multi_margin_loss_cpu_kernel(
   } else {
     accscalar_t sum = 0;
     auto output_acc = output.data_ptr<scalar_t>();
-    for (int64_t t = 0; t < nframe; t++) {
+    for (const auto t : c10::irange(nframe)) {
       const auto idx = target_index_checked(target_data, t, dim);
       sum += multi_margin_inner_sum_cpu(
           input_data, weight_data, p, margin, dim, idx);
@@ -91,16 +101,15 @@ void multi_margin_loss_out_cpu_template(
     const Tensor& input,
     const Tensor& target,
     int p,
-    Scalar margin,
-    const Tensor& weight,
+    const Scalar& margin,
+    const std::optional<Tensor>& weight,
     int64_t reduction) {
-  int64_t nframe, dim;
+  int64_t nframe = 0, dim = 0;
   const auto ndims = input.dim();
-  auto target_arg = TensorArg(target, "target", 2);
 
   TORCH_CHECK(p == 1 || p == 2, "only p == 1 and p == 2 supported");
 
-  multi_margin_loss_shape_check(nframe, dim, ndims, target_arg, input, target);
+  multi_margin_loss_shape_check(nframe, dim, ndims, input, target, weight);
 
   // produce a scalar output for 1d input
   if (reduction == Reduction::None && target.dim() > 0) {
@@ -114,13 +123,17 @@ void multi_margin_loss_out_cpu_template(
 
   auto input_contiguous = input.contiguous();
   auto target_contiguous = target.contiguous();
+  Tensor weight_contiguous;
+  if (weight && weight->defined()) {
+    weight_contiguous = weight->contiguous();
+  }
 
   AT_DISPATCH_FLOATING_TYPES(
       input.scalar_type(), "multi_margin_loss_cpu_kernel", [&] {
-        auto input_data = input_contiguous.data_ptr<scalar_t>();
-        auto target_data = target_contiguous.data_ptr<int64_t>();
+        auto input_data = input_contiguous.const_data_ptr<scalar_t>();
+        auto target_data = target_contiguous.const_data_ptr<int64_t>();
         auto weight_data =
-            weight.defined() ? weight.data_ptr<scalar_t>() : nullptr;
+            weight_contiguous.defined() ? weight_contiguous.const_data_ptr<scalar_t>() : nullptr;
         multi_margin_loss_cpu_kernel<scalar_t>(
             output,
             input_data,
@@ -138,21 +151,21 @@ template <typename scalar_t>
 static void multi_margin_loss_backward_cpu_kernel(
     scalar_t* grad_input_data,
     const Tensor& grad_output,
-    scalar_t* input_data,
-    int64_t* target_data,
+    const scalar_t* input_data,
+    const int64_t* target_data,
     int p,
     scalar_t margin,
     scalar_t g,
-    scalar_t* weight_data,
+    const scalar_t* weight_data,
     int64_t nframe,
     int64_t dim,
     int64_t reduction) {
   scalar_t* grad_input_row_data = grad_input_data;
-  for (int64_t t = 0; t < nframe; t++) {
+  for (const auto t : c10::irange(nframe)) {
     int64_t target_idx = target_index_checked(target_data, t, dim);
     scalar_t input_target = input_data[target_idx];
     scalar_t grad_input_target = 0;
-    for (int64_t d = 0; d < dim; d++) {
+    for (const auto d : c10::irange(dim)) {
       scalar_t z = margin - input_target + input_data[d];
       if (d == target_idx) {
         continue;
@@ -179,14 +192,14 @@ static void multi_margin_loss_backward_cpu_kernel(
     assert(
         reduction != Reduction::None || grad_output.dim() > 0 ||
         nframe == 1); // check 1d scalar fallback-case
-    const auto d = *grad_output.data_ptr<scalar_t>();
+    const auto d = *grad_output.const_data_ptr<scalar_t>();
     for (int64_t t = 0; t < nframe * dim; t++) {
       grad_input_data[t] *= d;
     }
   } else {
-    auto grad_output_acc = grad_output.accessor<scalar_t, 1>();
-    for (int64_t t = 0; t < nframe; t++) {
-      for (int64_t d = 0; d < dim; d++) {
+    auto grad_output_acc = grad_output.accessor<const scalar_t, 1>();
+    for (const auto t : c10::irange(nframe)) {
+      for (const auto d : c10::irange(dim)) {
         grad_input_data[t * dim + d] *= grad_output_acc[t];
       }
     }
@@ -199,16 +212,15 @@ void multi_margin_loss_backward_out_cpu_template(
     const Tensor& input,
     const Tensor& target,
     int p,
-    Scalar margin,
+    const Scalar& margin,
     const Tensor& weight,
     int64_t reduction) {
-  int64_t nframe, dim;
-  auto target_arg = TensorArg(target, "target", 2);
+  int64_t nframe = 0, dim = 0;
   const auto ndims = input.dim();
-  
+
   TORCH_CHECK(p == 1 || p == 2, "only p == 1 and p == 2 supported");
 
-  multi_margin_loss_shape_check(nframe, dim, ndims, target_arg, input, target);
+  multi_margin_loss_shape_check(nframe, dim, ndims, input, target, weight);
   grad_input.resize_as_(input);
   TORCH_CHECK(grad_input.is_contiguous(), "grad_input must be contiguous");
 
@@ -221,11 +233,11 @@ void multi_margin_loss_backward_out_cpu_template(
   auto weight_contiguous = weight.contiguous();
   AT_DISPATCH_FLOATING_TYPES(
       input.scalar_type(), "multi_margin_loss_backward_cpu_kernel", [&] {
-        auto grad_input_data = grad_input.data_ptr<scalar_t>();
-        auto input_data = input_contiguous.data_ptr<scalar_t>();
-        auto target_data = target_contiguous.data_ptr<int64_t>();
+        auto grad_input_data = grad_input.mutable_data_ptr<scalar_t>();
+        auto input_data = input_contiguous.const_data_ptr<scalar_t>();
+        auto target_data = target_contiguous.const_data_ptr<int64_t>();
         auto weight_data = weight_contiguous.defined()
-            ? weight_contiguous.data_ptr<scalar_t>()
+            ? weight_contiguous.const_data_ptr<scalar_t>()
             : nullptr;
         scalar_t g = reduction == Reduction::Mean
             ? static_cast<scalar_t>(1. / (nframe * dim))
@@ -250,9 +262,9 @@ void multi_margin_loss_backward_out_cpu_template(
 Tensor multi_margin_loss_cpu(
     const Tensor& input,
     const Tensor& target,
-    Scalar p,
-    Scalar margin,
-    const Tensor& weight,
+    const Scalar& p,
+    const Scalar& margin,
+    const std::optional<Tensor>& weight,
     int64_t reduction) {
   auto output = at::empty({0}, input.options());
   multi_margin_loss_out_cpu_template(
@@ -260,14 +272,13 @@ Tensor multi_margin_loss_cpu(
   return output;
 }
 
-Tensor& multi_margin_loss_cpu_out(
-    Tensor& output,
-    const Tensor& input,
+Tensor& multi_margin_loss_cpu_out(const Tensor& input,
     const Tensor& target,
-    Scalar p,
-    Scalar margin,
-    const Tensor& weight,
-    int64_t reduction) {
+    const Scalar& p,
+    const Scalar& margin,
+    const std::optional<Tensor>& weight,
+    int64_t reduction,
+    Tensor& output) {
   multi_margin_loss_out_cpu_template(
       output, input, target, p.toInt(), margin, weight, reduction);
   return output;
@@ -277,10 +288,13 @@ Tensor multi_margin_loss_cpu_backward(
     const Tensor& grad_output,
     const Tensor& input,
     const Tensor& target,
-    Scalar p,
-    Scalar margin,
-    const Tensor& weight,
+    const Scalar& p,
+    const Scalar& margin, const std::optional<Tensor>& weight_opt,
     int64_t reduction) {
+  // See [Note: hacky wrapper removal for optional tensor]
+  c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
+  const Tensor& weight = *weight_maybe_owned;
+
   auto grad_input = at::empty({0}, input.options());
   multi_margin_loss_backward_out_cpu_template(
       grad_input,
@@ -294,15 +308,17 @@ Tensor multi_margin_loss_cpu_backward(
   return grad_input;
 }
 
-Tensor& multi_margin_loss_cpu_backward_out(
-    Tensor& grad_input,
-    const Tensor& grad_output,
+Tensor& multi_margin_loss_cpu_backward_out(const Tensor& grad_output,
     const Tensor& input,
     const Tensor& target,
-    Scalar p,
-    Scalar margin,
-    const Tensor& weight,
-    int64_t reduction) {
+    const Scalar& p,
+    const Scalar& margin, const std::optional<Tensor>& weight_opt,
+    int64_t reduction,
+    Tensor& grad_input) {
+  // See [Note: hacky wrapper removal for optional tensor]
+  c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
+  const Tensor& weight = *weight_maybe_owned;
+
   multi_margin_loss_backward_out_cpu_template(
       grad_input,
       grad_output,
@@ -315,5 +331,4 @@ Tensor& multi_margin_loss_cpu_backward_out(
   return grad_input;
 }
 
-} // namespace native
-} // namespace at
+} // namespace at::native
