@@ -3,6 +3,7 @@
 #include <ATen/native/quantized/cpu/EmbeddingPackedParams.h>
 #include <ATen/native/quantized/cpu/fbgemm_utils.h>
 #include <ATen/native/quantized/cpu/qembeddingbag.h>
+#include <ATen/native/quantized/library.h>
 #include <torch/library.h>
 #ifdef USE_FBGEMM
 #include <fbgemm/Fbgemm.h>
@@ -28,8 +29,6 @@
 #include <arm_neon.h>
 #endif
 
-int register_embedding_params();
-
 namespace {
 
 // Fallback implementation when FBGEMM is not available.
