@@ -331,7 +331,7 @@ typedef void (*pytorch_q8gemm_dq_sparse_ukernel_function)(
     size_t output_channel_index,
     const struct pytorch_qnnp_conv_dynamic_quantization_params* quantization_params);
 
-typedef void (*pytorch_q8gemm_dq_sparse_packedA_ukernel_function)(
+typedef void (*pytorch_q8gemm_dq_sparse_packedA_w32_ukernel_function)(
     size_t mr,
     size_t nr,
     const uint8_t* a_packed,
@@ -344,6 +344,32 @@ typedef void (*pytorch_q8gemm_dq_sparse_packedA_ukernel_function)(
     size_t output_channel_index,
     const struct pytorch_qnnp_conv_dynamic_quantization_params* quantization_params);
 
+typedef void (*pytorch_q8gemm_dq_sparse_packedA_w16_ukernel_function)(
+    size_t mr,
+    size_t nr,
+    const uint8_t* a_packed,
+    const uint8_t* packed_w,
+    const uint16_t* w_row_ptr,
+    const uint16_t* w_block_ids_ptr,
+    const float* bias,
+    float* c,
+    size_t c_stride,
+    size_t output_channel_index,
+    const struct pytorch_qnnp_conv_dynamic_quantization_params* quantization_params);
+
+typedef void (*pytorch_q8gemm_dq_sparse_packedA_w8_ukernel_function)(
+    size_t mr,
+    size_t nr,
+    const uint8_t* a_packed,
+    const uint8_t* packed_w,
+    const uint8_t* w_row_ptr,
+    const uint8_t* w_block_ids_ptr,
+    const float* bias,
+    float* c,
+    size_t c_stride,
+    size_t output_channel_index,
+    const struct pytorch_qnnp_conv_dynamic_quantization_params* quantization_params);
+
 typedef void (*pytorch_q8gemm_sparse_packA_ukernel_function)(
     const size_t mr,
     const size_t K,
@@ -427,7 +453,7 @@ typedef void (*pytorch_hgemm_ukernel_function)(
     size_t c_stride,
     const struct pytorch_qnnp_fp16_clamping_params* clamping_params);
 
-typedef void (*pytorch_q8dwconv_up_ukernel_function)(
+typedef void (*pytorch_q8dwconv2d_up_ukernel_function)(
     size_t channels,
     size_t output_width,
     const uint8_t** input,
@@ -437,7 +463,7 @@ typedef void (*pytorch_q8dwconv_up_ukernel_function)(
     size_t output_increment,
     const union pytorch_qnnp_conv_quantization_params* quantization_params);
 
-typedef void (*pytorch_q8dwconv_mp_ukernel_function)(
+typedef void (*pytorch_q8dwconv2d_mp_ukernel_function)(
     size_t channels,
     size_t output_width,
     const uint8_t** input,
@@ -448,6 +474,19 @@ typedef void (*pytorch_q8dwconv_mp_ukernel_function)(
     size_t output_increment,
     const union pytorch_qnnp_conv_quantization_params* quantization_params);
 
+typedef void (*pytorch_q8dwconv3d_mp_ukernel_function)(
+    size_t channels,
+    size_t output_height,
+    size_t output_width,
+    const uint8_t** input,
+    const void* weights,
+    int32_t* buffer,
+    uint8_t* output,
+    size_t input_row_stride,
+    size_t input_col_stride,
+    size_t output_increment,
+    const union pytorch_qnnp_conv_quantization_params* quantization_params);
+
 typedef void (*pytorch_q8gavgpool_up_ukernel_function)(
     size_t m,
     size_t n,
@@ -532,12 +571,19 @@ struct pytorch_q8conv_parameters {
 
 struct pytorch_q8gemm_sparse_parameters {
   pytorch_q8gemm_dq_sparse_ukernel_function gemm_dq;
-  pytorch_q8gemm_dq_sparse_packedA_ukernel_function packedA_gemm_dq;
+  // w32, w16, and w8 refer to variants of the kernel which use uint32_t,
+  // uint16_t, and uint8_t datatype for row values/col indices respectively
+  pytorch_q8gemm_dq_sparse_packedA_w32_ukernel_function packedA_w32_gemm_dq;
+  pytorch_q8gemm_dq_sparse_packedA_w16_ukernel_function packedA_w16_gemm_dq;
+  pytorch_q8gemm_dq_sparse_packedA_w8_ukernel_function packedA_w8_gemm_dq;
   pytorch_q8gemm_sparse_packA_ukernel_function packA;
   uint8_t mr;
   uint8_t nr;
   uint8_t kr;
   uint8_t log2_mr;
+  uint8_t log2_row_block_size;
+  uint32_t row_block_size;
+  uint32_t col_block_size;
 };
 
 struct pytorch_q8conv_xzp_parameters {
@@ -550,15 +596,20 @@ struct pytorch_q8conv_xzp_parameters {
   size_t kthreshold;
 };
 
-struct pytorch_q8dwconv_up_parameters {
-  pytorch_q8dwconv_up_ukernel_function updw;
-  pytorch_q8dwconv_up_ukernel_function updw_per_channel;
+struct pytorch_q8dwconv2d_up_parameters {
+  pytorch_q8dwconv2d_up_ukernel_function updw;
+  pytorch_q8dwconv2d_up_ukernel_function updw_per_channel;
   uint8_t cr;
 };
 
-struct pytorch_q8dwconv_mp_parameters {
-  pytorch_q8dwconv_mp_ukernel_function mpdw;
-  pytorch_q8dwconv_mp_ukernel_function mpdw_per_channel;
+struct pytorch_q8dwconv2d_mp_parameters {
+  pytorch_q8dwconv2d_mp_ukernel_function mpdw;
+  pytorch_q8dwconv2d_mp_ukernel_function mpdw_per_channel;
+  uint8_t cr;
+};
+
+struct pytorch_q8dwconv3d_mp_parameters {
+  pytorch_q8dwconv3d_mp_ukernel_function mpdw;
   uint8_t cr;
 };
 
@@ -601,10 +652,12 @@ struct pytorch_x8zip_parameters {
 
 struct pytorch_qnnp_parameters {
   struct pytorch_q8conv_parameters q8conv;
-  struct pytorch_q8gemm_sparse_parameters q8gemm_sparse;
+  struct pytorch_q8gemm_sparse_parameters q8gemm_sparse_c1x4;
+  struct pytorch_q8gemm_sparse_parameters q8gemm_sparse_c8x1;
   struct pytorch_q8conv_xzp_parameters q8conv_xzp;
-  struct pytorch_q8dwconv_up_parameters q8dw9;
-  struct pytorch_q8dwconv_mp_parameters q8dw25;
+  struct pytorch_q8dwconv2d_up_parameters q8dw9;
+  struct pytorch_q8dwconv2d_mp_parameters q8dw25;
+  struct pytorch_q8dwconv3d_mp_parameters q8dw27;
   struct pytorch_q8sum_rows_parameters q8sum_rows;
   pytorch_q8vadd_ukernel_function q8vadd;
   struct pytorch_q8gavgpool_parameters q8gavgpool;
