@@ -50,14 +50,13 @@ TEST(MAX_POOLING_OP, unit_batch_many_channels_small_1xM_pool_with_padding) {
        channels += 3) {
     for (size_t poolSize = 3; poolSize <= pytorch_qnnp_params.u8maxpool.mr;
          poolSize++) {
-      for (size_t paddingLeft = 0; paddingLeft <= 1; paddingLeft++) {
+      for (size_t paddingWidth = 0; paddingWidth <= 1; paddingWidth++) {
         for (size_t paddingRight = 0; paddingRight <= 1; paddingRight++) {
           MaxPoolingOperatorTester()
               .batchSize(1)
               .inputHeight(2)
               .inputWidth(poolSize + 2)
-              .paddingLeft(paddingLeft)
-              .paddingRight(paddingRight)
+              .paddingWidth(paddingWidth)
               .poolingHeight(1)
               .poolingWidth(poolSize)
               .channels(channels)
@@ -134,14 +133,13 @@ TEST(MAX_POOLING_OP, unit_batch_many_channels_small_Mx1_pool_with_padding) {
        channels += 3) {
     for (size_t poolSize = 2; poolSize <= pytorch_qnnp_params.u8maxpool.mr;
          poolSize++) {
-      for (size_t paddingTop = 0; paddingTop <= 1; paddingTop++) {
+      for (size_t paddingHeight = 0; paddingHeight <= 1; paddingHeight++) {
         for (size_t paddingBottom = 0; paddingBottom <= 1; paddingBottom++) {
           MaxPoolingOperatorTester()
               .batchSize(1)
               .inputHeight(poolSize + 1)
               .inputWidth(3)
-              .paddingTop(paddingTop)
-              .paddingBottom(paddingBottom)
+              .paddingHeight(paddingHeight)
               .poolingHeight(poolSize)
               .poolingWidth(1)
               .channels(channels)
@@ -335,14 +333,13 @@ TEST(MAX_POOLING_OP, unit_batch_many_channels_large_1xM_pool_with_padding) {
        channels += 3) {
     for (size_t poolSize = 3; poolSize <= pytorch_qnnp_params.u8maxpool.mr;
          poolSize++) {
-      for (size_t paddingLeft = 0; paddingLeft <= 1; paddingLeft++) {
+      for (size_t paddingWidth = 0; paddingWidth <= 1; paddingWidth++) {
         for (size_t paddingRight = 0; paddingRight <= 1; paddingRight++) {
           MaxPoolingOperatorTester()
               .batchSize(1)
               .inputHeight(2)
               .inputWidth(poolSize + 2)
-              .paddingLeft(paddingLeft)
-              .paddingRight(paddingRight)
+              .paddingWidth(paddingWidth)
               .poolingHeight(1)
               .poolingWidth(poolSize)
               .channels(channels)
@@ -423,14 +420,13 @@ TEST(MAX_POOLING_OP, unit_batch_many_channels_large_Mx1_pool_with_padding) {
     for (size_t poolSize = pytorch_qnnp_params.u8maxpool.mr; poolSize <=
          pytorch_qnnp_params.u8maxpool.mr + pytorch_qnnp_params.u8maxpool.qr;
          poolSize++) {
-      for (size_t paddingTop = 0; paddingTop <= 1; paddingTop++) {
+      for (size_t paddingHeight = 0; paddingHeight <= 1; paddingHeight++) {
         for (size_t paddingBottom = 0; paddingBottom <= 1; paddingBottom++) {
           MaxPoolingOperatorTester()
               .batchSize(1)
               .inputHeight(poolSize + 1)
               .inputWidth(3)
-              .paddingTop(paddingTop)
-              .paddingBottom(paddingBottom)
+              .paddingHeight(paddingHeight)
               .poolingHeight(poolSize)
               .poolingWidth(1)
               .channels(channels)
@@ -627,14 +623,13 @@ TEST(MAX_POOLING_OP, unit_batch_few_channels_1xM_pool_with_padding) {
        channels++) {
     for (size_t poolSize = 3; poolSize <= pytorch_qnnp_params.u8maxpool.mr;
          poolSize++) {
-      for (size_t paddingLeft = 0; paddingLeft <= 1; paddingLeft++) {
+      for (size_t paddingWidth = 0; paddingWidth <= 1; paddingWidth++) {
         for (size_t paddingRight = 0; paddingRight <= 1; paddingRight++) {
           MaxPoolingOperatorTester()
               .batchSize(1)
               .inputHeight(2)
               .inputWidth(poolSize + 2)
-              .paddingLeft(paddingLeft)
-              .paddingRight(paddingRight)
+              .paddingWidth(paddingWidth)
               .poolingHeight(1)
               .poolingWidth(poolSize)
               .channels(channels)
@@ -707,14 +702,13 @@ TEST(MAX_POOLING_OP, unit_batch_few_channels_Mx1_pool_with_padding) {
        channels++) {
     for (size_t poolSize = 2; poolSize <= 2 * pytorch_qnnp_params.u8maxpool.kr;
          poolSize++) {
-      for (size_t paddingTop = 0; paddingTop <= 1; paddingTop++) {
+      for (size_t paddingHeight = 0; paddingHeight <= 1; paddingHeight++) {
         for (size_t paddingBottom = 0; paddingBottom <= 1; paddingBottom++) {
           MaxPoolingOperatorTester()
               .batchSize(1)
               .inputHeight(poolSize + 1)
               .inputWidth(3)
-              .paddingTop(paddingTop)
-              .paddingBottom(paddingBottom)
+              .paddingHeight(paddingHeight)
               .poolingHeight(poolSize)
               .poolingWidth(1)
               .channels(channels)
