@@ -9,8 +9,8 @@
 
 #include <gtest/gtest.h>
 
-namespace qnnpack {
-namespace testing {
+
+namespace qnnpack::testing {
 
 enum class Mode {
   Static,
@@ -23,13 +23,13 @@ enum class Mode {
   }
 
 #define _STATIC_TEST(TestClass, test_name, test_body)                   \
-  _MAKE_TEST(TestClass, test_name##_static, test_body, Mode::Static)
+  _MAKE_TEST(TestClass, test_name##_static, test_body, qnnpack::testing::Mode::Static)
 
 #define _RUNTIME_TEST(TestClass, test_name, test_body)                  \
-  _MAKE_TEST(TestClass, test_name##_runtime, test_body, Mode::Runtime)
+  _MAKE_TEST(TestClass, test_name##_runtime, test_body, qnnpack::testing::Mode::Runtime)
 
 #define _STATIC_AND_RUNTIME_TEST(TestClass, test_name, test_body) \
   _STATIC_TEST(TestClass, test_name, test_body)                   \
   _RUNTIME_TEST(TestClass, test_name, test_body)
 
-}}  // namespace qnnpack::testing
+} // namespace qnnpack::testing
