@@ -1,6 +1,6 @@
 #pragma once
 
-#include <ATen/native/vulkan/api/Common.h>
+#ifdef USE_VULKAN_API
 
 #include <ATen/native/vulkan/api/Adapter.h>
 #include <ATen/native/vulkan/api/Command.h>
@@ -10,3 +10,8 @@
 #include <ATen/native/vulkan/api/Resource.h>
 #include <ATen/native/vulkan/api/Runtime.h>
 #include <ATen/native/vulkan/api/Shader.h>
+#include <ATen/native/vulkan/api/ShaderRegistry.h>
+#include <ATen/native/vulkan/api/Tensor.h>
+#include <ATen/native/vulkan/api/Utils.h>
+
+#endif /* USE_VULKAN_API */
