@@ -1,6 +1,8 @@
+#include <gtest/gtest.h>
+
 #include <ATen/ATen.h>
 #include <ATen/Parallel.h>
-#include <test/cpp/tensorexpr/test_base.h>
+#include <c10/util/irange.h>
 #include <thread>
 
 
@@ -8,17 +10,17 @@
 // numbers of threads set and also whether the scheduler
 // will throw an exception when multiple threads call
 // their first parallel construct.
-void test(int given_num_threads) {
+static void test(int given_num_threads) {
   auto t = at::ones({1000 * 1000}, at::CPU(at::kFloat));
   ASSERT_TRUE(given_num_threads >= 0);
   ASSERT_EQ(at::get_num_threads(), given_num_threads);
   auto t_sum = t.sum();
-  for (int i = 0; i < 1000; ++i) {
+  for ([[maybe_unused]] const auto i : c10::irange(1000)) {
     t_sum = t_sum + t.sum();
   }
 }
 
-int main() {
+TEST(ThreadInitTest, ThreadInit) {
   at::init_num_threads();
 
   at::set_num_threads(4);
@@ -31,13 +33,11 @@ int main() {
 
   #if !AT_PARALLEL_NATIVE
   at::set_num_threads(5);
-  ASSERT_TRUE(at::get_num_threads() == 5);
+  ASSERT_EQ(at::get_num_threads(), 5);
   #endif
 
   // test inter-op settings
   at::set_num_interop_threads(5);
   ASSERT_EQ(at::get_num_interop_threads(), 5);
   ASSERT_ANY_THROW(at::set_num_interop_threads(6));
-
-  return 0;
 }
