@@ -12,8 +12,16 @@
 #error "AT_MKLDNN_ENABLED should not be visible in public headers"
 #endif
 
+#ifdef AT_MKLDNN_ACL_ENABLED
+#error "AT_MKLDNN_ACL_ENABLED should not be visible in public headers"
+#endif
+
 #ifdef CAFFE2_STATIC_LINK_CUDA
 #error "CAFFE2_STATIC_LINK_CUDA should not be visible in public headers"
 #endif
 
-auto main() -> int {}
+#include <gtest/gtest.h>
+
+TEST(VerifyApiVisibility, Test) {
+  ASSERT_EQ(1, 1);
+}
