@@ -138,6 +138,10 @@ class TorchBenchmarkRunner(BenchmarkRunner):
     def skip_models_for_cpu(self):
         return self._skip["device"]["cpu"]
 
+    @property
+    def skip_models_for_cpu_aarch64(self):
+        return self._skip["device"]["cpu_aarch64"]
+
     @property
     def skip_models_for_cuda(self):
         return self._skip["device"]["cuda"]
@@ -378,6 +382,22 @@ class TorchBenchmarkRunner(BenchmarkRunner):
         if self.args.trace_on_xla:
             # work around for: https://github.com/pytorch/xla/issues/4174
             import torch_xla  # noqa: F401
+
+        # Turning off kv cache for torchbench models. This is not the right
+        # thing to do, but the torchbench models are way outdated, and since we
+        # are using torchbench pt2 dashboard to track regressions (rather than
+        # improving performance), we are just setting the kv cache to false.
+        # Real transformers benchmarks will be added soon using a different
+        # infra.
+        if (
+            model_name.startswith("hf")
+            and hasattr(model, "config")
+            and hasattr(model.config, "use_cache")
+        ):
+            model.config.use_cache = False
+        if model_name == "hf_T5_generate":
+            model.model.config.use_cache = False
+
         self.validate_model(model, example_inputs)
         return device, benchmark.name, model, example_inputs, batch_size
 
@@ -438,6 +458,8 @@ class TorchBenchmarkRunner(BenchmarkRunner):
         if self.args.bfloat16:
             if name in self._tolerance["higher_bf16"]:
                 return 1e-2, cosine
+            elif current_device == "xpu" and name in self._tolerance["higher_bf16_xpu"]:
+                return 8 * 1e-2, cosine
 
         if is_training and (current_device == "cuda" or current_device == "xpu"):
             tolerance = 1e-3
