@@ -1,7 +1,9 @@
-import operator_benchmark as op_bench
-import torch
 import numpy
 
+import operator_benchmark as op_bench
+
+import torch
+
 
 """Microbenchmarks for index_select operator."""
 
@@ -17,9 +19,9 @@ index_select_configs_short = op_bench.config_list(
         [512, 512, 2, 1],
     ],
     cross_product_configs={
-        'device': ['cpu', 'cuda'],
+        "device": ["cpu", "cuda"],
     },
-    tags=["short"]
+    tags=["short"],
 )
 
 
@@ -28,8 +30,8 @@ index_select_configs_long = op_bench.cross_product_configs(
     N=[128, 1024],
     K=[1, 2],
     dim=[1],
-    device=['cpu', 'cuda'],
-    tags=["long"]
+    device=["cpu", "cuda"],
+    tags=["long"],
 )
 
 
@@ -40,8 +42,10 @@ class IndexSelectBenchmark(op_bench.TorchBenchmarkBase):
         index_dim = numpy.random.randint(0, N)
         self.inputs = {
             "input_one": torch.rand(M, N, K, device=device),
-            "dim" : dim,
-            "index" : torch.tensor(numpy.random.randint(0, max_val, index_dim), device=device),
+            "dim": dim,
+            "index": torch.tensor(
+                numpy.random.randint(0, max_val, index_dim), device=device
+            ),
         }
         self.set_module_name("index_select")
 
@@ -49,8 +53,9 @@ class IndexSelectBenchmark(op_bench.TorchBenchmarkBase):
         return torch.index_select(input_one, dim, index)
 
 
-op_bench.generate_pt_test(index_select_configs_short + index_select_configs_long,
-                          IndexSelectBenchmark)
+op_bench.generate_pt_test(
+    index_select_configs_short + index_select_configs_long, IndexSelectBenchmark
+)
 
 
 if __name__ == "__main__":
