@@ -265,7 +265,7 @@ class _BaseDataSparsiferTestCase(TestCase):
 class _NormDataSparsifierTestCase(_BaseDataSparsiferTestCase):
     r"""This helper test class takes in any supported type of and runs some tests.
     This inherits the TestBaseDataSparsifierRuner wherein some functions are
-    over-ridden to take accomodate the specific sparsifier.
+    over-ridden to take accommodate the specific sparsifier.
     TODO: Change the structure by creating a separate test case class for each
           member function
     """
@@ -770,7 +770,7 @@ class TestQuantizationUtils(TestCase):
 
         # higher threshold as quantization occurs before sparsity
         threshold = (
-            1  # zero points seem to have higher magnitude with sparsity occuring after
+            1  # zero points seem to have higher magnitude with sparsity occurring after
         )
 
         sl_emb1 = (torch.abs(dequant_emb1) < threshold).float().mean()
