@@ -1,6 +1,7 @@
 #include <gtest/gtest.h>
 #include <test/cpp/api/support.h>
 
+#include <c10/util/irange.h>
 #include <torch/torch.h>
 
 #include <cmath>
@@ -75,6 +76,14 @@ TEST(TensorTest, ToTensorAndTensorAttributes) {
   REQUIRE_TENSOR_OPTIONS(at::kCPU, -1, at::kInt, at::kStrided);
 }
 
+TEST(TensorTest, TensorAttributes) {
+  std::vector<float> v = {1.0, 2.0, 3.0};
+  auto options = c10::requires_grad(true);
+  auto tensor = at::from_blob(v.data(), {3}, options);
+  REQUIRE_TENSOR_OPTIONS(at::kCPU, -1, at::kFloat, at::kStrided);
+  ASSERT_TRUE(tensor.requires_grad());
+}
+
 // Not currently supported.
 // TEST(TensorTest, ToLayout) {
 //   auto tensor = at::empty({3, 4});
@@ -97,6 +106,7 @@ TEST(TensorTest, ToOptionsWithRequiresGrad) {
     ASSERT_TRUE(tensor.requires_grad());
 
     // Throws if requires_grad is set in TensorOptions
+    // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
     ASSERT_THROW(
         tensor.to(at::TensorOptions().requires_grad(true)), c10::Error);
 
@@ -113,6 +123,7 @@ TEST(TensorTest, ToOptionsWithRequiresGrad) {
     ASSERT_FALSE(tensor.requires_grad());
 
     // Throws if requires_grad is set in TensorOptions
+    // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
     ASSERT_THROW(
         tensor.to(at::TensorOptions().requires_grad(true)), c10::Error);
 
@@ -176,7 +187,9 @@ TEST(TensorTest, AtTensorCtorScalar) {
   ASSERT_EQ(tensor.dtype(), at::kComplexFloat);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<float>(1.5, 2.0)));
 
-  tensor = at::tensor(c10::complex<float>(1.0, 2.0), at::dtype(at::kComplexFloat)) + 0.5;
+  tensor =
+      at::tensor(c10::complex<float>(1.0, 2.0), at::dtype(at::kComplexFloat)) +
+      0.5;
   ASSERT_EQ(tensor.numel(), 1);
   ASSERT_EQ(tensor.dtype(), at::kComplexFloat);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<float>(1.5, 2.0)));
@@ -186,7 +199,9 @@ TEST(TensorTest, AtTensorCtorScalar) {
   ASSERT_EQ(tensor.dtype(), at::kComplexDouble);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<double>(1.5, 2.0)));
 
-  tensor = at::tensor(c10::complex<float>(1.0, 2.0), at::dtype(at::kComplexDouble)) + 0.5;
+  tensor =
+      at::tensor(c10::complex<float>(1.0, 2.0), at::dtype(at::kComplexDouble)) +
+      0.5;
   ASSERT_EQ(tensor.numel(), 1);
   ASSERT_EQ(tensor.dtype(), at::kComplexDouble);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<double>(1.5, 2.0)));
@@ -214,14 +229,20 @@ TEST(TensorTest, AtTensorCtorSingleDim) {
   ASSERT_TRUE(almost_equal(tensor[1], 2.25));
   ASSERT_TRUE(almost_equal(tensor[2], 3.125));
 
-  tensor = at::tensor({c10::complex<float>(1.5, 0.15), c10::complex<float>(1.5, 0.15), c10::complex<float>(3.125, 0.3125)});
+  tensor = at::tensor(
+      {c10::complex<float>(1.5, 0.15),
+       c10::complex<float>(1.5, 0.15),
+       c10::complex<float>(3.125, 0.3125)});
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.dtype(), at::kComplexFloat);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<float>(1.5, 0.15)));
   ASSERT_TRUE(almost_equal(tensor[1], c10::complex<float>(1.5, 0.15)));
   ASSERT_TRUE(almost_equal(tensor[2], c10::complex<float>(3.125, 0.3125)));
 
-  tensor = at::tensor({c10::complex<double>(1.5, 0.15), c10::complex<double>(1.5, 0.15), c10::complex<double>(3.125, 0.3125)});
+  tensor = at::tensor(
+      {c10::complex<double>(1.5, 0.15),
+       c10::complex<double>(1.5, 0.15),
+       c10::complex<double>(3.125, 0.3125)});
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.dtype(), at::kComplexDouble);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<double>(1.5, 0.15)));
@@ -243,14 +264,20 @@ TEST(TensorTest, AtTensorCtorSingleDim) {
   ASSERT_TRUE(almost_equal(tensor[1], 2.25));
   ASSERT_TRUE(almost_equal(tensor[2], 3.125));
 
-  tensor = at::tensor(std::vector<c10::complex<float>>({c10::complex<float>(1.5, 0.15), c10::complex<float>(1.5, 0.15), c10::complex<float>(3.125, 0.3125)}));
+  tensor = at::tensor(std::vector<c10::complex<float>>(
+      {c10::complex<float>(1.5, 0.15),
+       c10::complex<float>(1.5, 0.15),
+       c10::complex<float>(3.125, 0.3125)}));
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.dtype(), at::kComplexFloat);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<float>(1.5, 0.15)));
   ASSERT_TRUE(almost_equal(tensor[1], c10::complex<float>(1.5, 0.15)));
   ASSERT_TRUE(almost_equal(tensor[2], c10::complex<float>(3.125, 0.3125)));
 
-  tensor = at::tensor(std::vector<c10::complex<double>>({c10::complex<double>(1.5, 0.15), c10::complex<double>(1.5, 0.15), c10::complex<double>(3.125, 0.3125)}));
+  tensor = at::tensor(std::vector<c10::complex<double>>(
+      {c10::complex<double>(1.5, 0.15),
+       c10::complex<double>(1.5, 0.15),
+       c10::complex<double>(3.125, 0.3125)}));
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.dtype(), at::kComplexDouble);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<double>(1.5, 0.15)));
@@ -261,7 +288,7 @@ TEST(TensorTest, AtTensorCtorSingleDim) {
   tensor = at::tensor(v);
   ASSERT_EQ(tensor.numel(), v.size());
   ASSERT_EQ(tensor.dtype(), at::kInt);
-  for (size_t i = 0; i < v.size(); ++i) {
+  for (const auto i : c10::irange(v.size())) {
     ASSERT_TRUE(exactly_equal(tensor[i], v.at(i)));
   }
 
@@ -269,24 +296,32 @@ TEST(TensorTest, AtTensorCtorSingleDim) {
   tensor = at::tensor(w);
   ASSERT_EQ(tensor.numel(), w.size());
   ASSERT_EQ(tensor.dtype(), at::kDouble);
-  for (size_t i = 0; i < w.size(); ++i) {
+  for (const auto i : c10::irange(w.size())) {
     ASSERT_TRUE(almost_equal(tensor[i], w.at(i)));
   }
 
   std::vector<c10::complex<double>> x = {
-    {1.1, -1.1}, {2.2, -2.2}, {3.3, -3.3}, {4.4, -4.4}, {5.5, -5.5},
-    {6.6, -6.6}, {7.7, -7.7}, {8.8, -8.8}, {9.9, -9.9}, {10.0, -10.0}
-  };
+      {1.1, -1.1},
+      {2.2, -2.2},
+      {3.3, -3.3},
+      {4.4, -4.4},
+      {5.5, -5.5},
+      {6.6, -6.6},
+      {7.7, -7.7},
+      {8.8, -8.8},
+      {9.9, -9.9},
+      {10.0, -10.0}};
   tensor = at::tensor(x);
   ASSERT_EQ(tensor.numel(), x.size());
   ASSERT_EQ(tensor.dtype(), at::kComplexDouble);
-  for (size_t i = 0; i < x.size(); ++i) {
+  for (const auto i : c10::irange(x.size())) {
     ASSERT_TRUE(almost_equal(tensor[i], x.at(i)));
   }
 }
 
 TEST(TensorTest, AtTensorCastRealToComplex) {
-  auto tensor = at::tensor(std::vector<double>({1.5, 2.5, 3.5}), at::kComplexDouble);
+  auto tensor =
+      at::tensor(std::vector<double>({1.5, 2.5, 3.5}), at::kComplexDouble);
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.dtype(), at::kComplexDouble);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<double>(1.5)));
@@ -308,16 +343,21 @@ TEST(TensorTest, AtTensorCastRealToComplex) {
 
 TEST(TensorTest, AtTensorCastComplexToRealErrorChecks) {
   {
-    ASSERT_THROWS_WITH(at::tensor(c10::complex<float>(0.1, 0.2), at::kFloat),
-      "\"tensor_cpu\" not implemented for 'Float'");
+    ASSERT_THROWS_WITH(
+        at::tensor(c10::complex<float>(0.1, 0.2), at::kFloat),
+        "\"tensor_cpu\" not implemented for 'Float'");
   }
   {
-    ASSERT_THROWS_WITH(at::tensor({c10::complex<float>(0.1, 0.2)}, at::kFloat),
-      "\"tensor_cpu\" not implemented for 'Float'");
+    ASSERT_THROWS_WITH(
+        at::tensor({c10::complex<float>(0.1, 0.2)}, at::kFloat),
+        "\"tensor_cpu\" not implemented for 'Float'");
   }
   {
-    ASSERT_THROWS_WITH(at::tensor(std::vector<c10::complex<float>>{c10::complex<float>(0.1, 0.2)}, at::kFloat),
-      "\"tensor_cpu\" not implemented for 'Float'");
+    ASSERT_THROWS_WITH(
+        at::tensor(
+            std::vector<c10::complex<float>>{c10::complex<float>(0.1, 0.2)},
+            at::kFloat),
+        "\"tensor_cpu\" not implemented for 'Float'");
   }
 }
 
@@ -329,7 +369,8 @@ TEST(TensorTest, TorchTensorCtorScalarIntegralType) {
   ASSERT_EQ(tensor.item<int64_t>(), 123);
 }
 
-void test_TorchTensorCtorScalarFloatingType_expected_dtype(c10::ScalarType default_dtype) {
+void test_TorchTensorCtorScalarFloatingType_expected_dtype(
+    c10::ScalarType default_dtype) {
   AutoDefaultDtypeMode dtype_mode(default_dtype);
 
   auto tensor = torch::tensor(123.456f);
@@ -352,8 +393,10 @@ void test_TorchTensorCtorScalarFloatingType_expected_dtype(c10::ScalarType defau
 }
 
 TEST(TensorTest, TorchTensorCtorScalarFloatingType) {
-  test_TorchTensorCtorScalarFloatingType_expected_dtype(/*default_dtype=*/torch::kFloat);
-  test_TorchTensorCtorScalarFloatingType_expected_dtype(/*default_dtype=*/torch::kDouble);
+  test_TorchTensorCtorScalarFloatingType_expected_dtype(
+      /*default_dtype=*/torch::kFloat);
+  test_TorchTensorCtorScalarFloatingType_expected_dtype(
+      /*default_dtype=*/torch::kDouble);
 }
 
 TEST(TensorTest, TorchTensorCtorScalarBoolType) {
@@ -412,7 +455,8 @@ TEST(TensorTest, TorchTensorCtorSingleDimIntegralType) {
   ASSERT_TRUE(exactly_equal(tensor[2], 3));
 }
 
-void test_TorchTensorCtorSingleDimFloatingType_expected_dtype(c10::ScalarType default_dtype) {
+void test_TorchTensorCtorSingleDimFloatingType_expected_dtype(
+    c10::ScalarType default_dtype) {
   AutoDefaultDtypeMode dtype_mode(default_dtype);
 
   auto tensor = torch::tensor({1.5, 2.25, 3.125});
@@ -424,7 +468,6 @@ void test_TorchTensorCtorSingleDimFloatingType_expected_dtype(c10::ScalarType de
   ASSERT_TRUE(almost_equal(tensor[2], 3.125));
 
   tensor = torch::tensor({1.5f, 2.25f, 3.125f});
-  ASSERT_TRUE(tensor.is_variable());
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({3}));
   ASSERT_EQ(tensor.dtype(), default_dtype);
@@ -433,7 +476,6 @@ void test_TorchTensorCtorSingleDimFloatingType_expected_dtype(c10::ScalarType de
   ASSERT_TRUE(almost_equal(tensor[2], 3.125f));
 
   tensor = torch::tensor(at::ArrayRef<float>({1.5f, 2.25f, 3.125f}));
-  ASSERT_TRUE(tensor.is_variable());
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.dtype(), default_dtype);
   ASSERT_TRUE(almost_equal(tensor[0], 1.5));
@@ -441,7 +483,6 @@ void test_TorchTensorCtorSingleDimFloatingType_expected_dtype(c10::ScalarType de
   ASSERT_TRUE(almost_equal(tensor[2], 3.125));
 
   tensor = torch::tensor(std::vector<float>({1.5f, 2.25f, 3.125f}));
-  ASSERT_TRUE(tensor.is_variable());
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({3}));
   ASSERT_EQ(tensor.dtype(), default_dtype);
@@ -466,8 +507,10 @@ void test_TorchTensorCtorSingleDimFloatingType_expected_dtype(c10::ScalarType de
 }
 
 TEST(TensorTest, TorchTensorCtorSingleDimFloatingType) {
-  test_TorchTensorCtorSingleDimFloatingType_expected_dtype(/*default_dtype=*/torch::kFloat);
-  test_TorchTensorCtorSingleDimFloatingType_expected_dtype(/*default_dtype=*/torch::kDouble);
+  test_TorchTensorCtorSingleDimFloatingType_expected_dtype(
+      /*default_dtype=*/torch::kFloat);
+  test_TorchTensorCtorSingleDimFloatingType_expected_dtype(
+      /*default_dtype=*/torch::kDouble);
 }
 
 TEST(TensorTest, TorchTensorCtorSingleDimBoolType) {
@@ -493,74 +536,91 @@ TEST(TensorTest, TorchTensorCtorMultiDimIntegralType) {
     auto tensor = torch::tensor({{1, 2}});
     ASSERT_EQ(tensor.dtype(), torch::kLong);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 2}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
     auto tensor = torch::tensor({{1}, {2}});
     ASSERT_EQ(tensor.dtype(), torch::kLong);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({2, 1}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
     auto tensor = torch::tensor({{{1, 2}}});
     ASSERT_EQ(tensor.dtype(), torch::kLong);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 1, 2}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
     auto tensor = torch::tensor({{{1}, {2}}});
     ASSERT_EQ(tensor.dtype(), torch::kLong);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 2, 1}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
     auto tensor = torch::tensor({{1, 2}, {3, 4}});
     ASSERT_EQ(tensor.dtype(), torch::kLong);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({2, 2}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 5, torch::kLong).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 5, torch::kLong).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
     auto tensor = torch::tensor({{{{{{{{{{1}}}}}}}}}});
     ASSERT_EQ(tensor.dtype(), torch::kLong);
-    ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 1, 1, 1, 1, 1, 1, 1, 1, 1}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::full({1}, 1, torch::kLong).view(tensor.sizes())));
+    ASSERT_EQ(
+        tensor.sizes(), std::vector<int64_t>({1, 1, 1, 1, 1, 1, 1, 1, 1, 1}));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::full({1}, 1, torch::kLong).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
     auto tensor = torch::tensor({{{{{{{{{{1, 2}}}}}}}}}});
     ASSERT_EQ(tensor.dtype(), torch::kLong);
-    ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 1, 1, 1, 1, 1, 1, 1, 1, 2}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
+    ASSERT_EQ(
+        tensor.sizes(), std::vector<int64_t>({1, 1, 1, 1, 1, 1, 1, 1, 1, 2}));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 3, torch::kLong).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
 }
 
-void test_TorchTensorCtorMultiDimFloatingType_expected_dtype(c10::ScalarType default_dtype) {
+void test_TorchTensorCtorMultiDimFloatingType_expected_dtype(
+    c10::ScalarType default_dtype) {
   AutoDefaultDtypeMode dtype_mode(default_dtype);
   {
     auto tensor = torch::tensor({{1.0, 2.0}});
     ASSERT_EQ(tensor.dtype(), default_dtype);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 2}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 3, default_dtype).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 3, default_dtype).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
-    auto tensor = torch::tensor({{{{{{{{1.0, 2.0, 3.0}}}}}, {{{{{4.0, 5.0, 6.0}}}}}, {{{{{7.0, 8.0, 9.0}}}}}}}});
+    auto tensor = torch::tensor(
+        {{{{{{{{1.0, 2.0, 3.0}}}}},
+           {{{{{4.0, 5.0, 6.0}}}}},
+           {{{{{7.0, 8.0, 9.0}}}}}}}});
     ASSERT_EQ(tensor.dtype(), default_dtype);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 1, 3, 1, 1, 1, 1, 3}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 10, default_dtype).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 10, default_dtype).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
 }
 
 TEST(TensorTest, TorchTensorCtorMultiDimFloatingType) {
-  test_TorchTensorCtorMultiDimFloatingType_expected_dtype(/*default_dtype=*/torch::kFloat);
-  test_TorchTensorCtorMultiDimFloatingType_expected_dtype(/*default_dtype=*/torch::kDouble);
+  test_TorchTensorCtorMultiDimFloatingType_expected_dtype(
+      /*default_dtype=*/torch::kFloat);
+  test_TorchTensorCtorMultiDimFloatingType_expected_dtype(
+      /*default_dtype=*/torch::kDouble);
 }
 
 TEST(TensorTest, TorchTensorCtorMultiDimBoolType) {
@@ -591,43 +651,52 @@ TEST(TensorTest, TorchTensorCtorMultiDimWithOptions) {
     auto tensor = torch::tensor({{1, 2}}, torch::dtype(torch::kInt));
     ASSERT_EQ(tensor.dtype(), torch::kInt);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 2}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 3, torch::kInt).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 3, torch::kInt).view(tensor.sizes())));
     ASSERT_FALSE(tensor.requires_grad());
   }
   {
-    auto tensor = torch::tensor({{1, 2}, {3, 4}}, torch::dtype(torch::kFloat).requires_grad(true));
+    auto tensor = torch::tensor(
+        {{1, 2}, {3, 4}}, torch::dtype(torch::kFloat).requires_grad(true));
     ASSERT_EQ(tensor.dtype(), torch::kFloat);
     ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({2, 2}));
-    ASSERT_TRUE(torch::allclose(tensor, torch::arange(1, 5, torch::kFloat).view(tensor.sizes())));
+    ASSERT_TRUE(torch::allclose(
+        tensor, torch::arange(1, 5, torch::kFloat).view(tensor.sizes())));
     ASSERT_TRUE(tensor.requires_grad());
   }
 }
 
 TEST(TensorTest, TorchTensorCtorMultiDimErrorChecks) {
   {
-    ASSERT_THROWS_WITH(torch::tensor({{{2, 3, 4}, {{5, 6}, {7}}}}),
-      "Expected all sub-lists to have sizes: 2 (e.g. {5, 6}), but got sub-list {7} with sizes: 1");
+    ASSERT_THROWS_WITH(
+        torch::tensor({{{2, 3, 4}, {{5, 6}, {7}}}}),
+        "Expected all sub-lists to have sizes: 2 (e.g. {5, 6}), but got sub-list {7} with sizes: 1");
   }
   {
-    ASSERT_THROWS_WITH(torch::tensor({{{1, 2.0}, {1, 2.0}}}),
-      "Expected all elements of the tensor to have the same scalar type: Int, but got element of scalar type: Double");
+    ASSERT_THROWS_WITH(
+        torch::tensor({{{1, 2.0}, {1, 2.0}}}),
+        "Expected all elements of the tensor to have the same scalar type: Int, but got element of scalar type: Double");
   }
   {
-    ASSERT_THROWS_WITH(torch::tensor({{{true, 2.0, 3}, {true, 2.0, 3}}}),
-      "Expected all elements of the tensor to have the same scalar type: Bool, but got element of scalar type: Double");
+    ASSERT_THROWS_WITH(
+        torch::tensor({{{true, 2.0, 3}, {true, 2.0, 3}}}),
+        "Expected all elements of the tensor to have the same scalar type: Bool, but got element of scalar type: Double");
   }
   {
-    ASSERT_THROWS_WITH(torch::tensor({{{true}, {2}}}),
-      "Expected all elements of the tensor to have the same scalar type: Bool, but got element of scalar type: Int");
+    ASSERT_THROWS_WITH(
+        torch::tensor({{{true}, {2}}}),
+        "Expected all elements of the tensor to have the same scalar type: Bool, but got element of scalar type: Int");
   }
   {
-    ASSERT_THROWS_WITH(torch::tensor({{{true, 2}}}),
-      "Expected all elements of the tensor to have the same scalar type: Bool, but got element of scalar type: Int");
+    ASSERT_THROWS_WITH(
+        torch::tensor({{{true, 2}}}),
+        "Expected all elements of the tensor to have the same scalar type: Bool, but got element of scalar type: Int");
   }
 }
 
 TEST(TensorTest, TorchTensorCastRealToComplex) {
-  auto tensor = torch::tensor(std::vector<double>({1.5, 2.5, 3.5}), torch::kComplexDouble);
+  auto tensor = torch::tensor(
+      std::vector<double>({1.5, 2.5, 3.5}), torch::kComplexDouble);
   ASSERT_EQ(tensor.numel(), 3);
   ASSERT_EQ(tensor.dtype(), torch::kComplexDouble);
   ASSERT_TRUE(almost_equal(tensor[0], c10::complex<double>(1.5)));
@@ -649,40 +718,56 @@ TEST(TensorTest, TorchTensorCastRealToComplex) {
 
 TEST(TensorTest, TorchTensorCastComplexToRealErrorChecks) {
   {
-    ASSERT_THROWS_WITH(torch::tensor(c10::complex<float>(0.1, 0.2), torch::kFloat),
-      "value cannot be converted to type float without overflow");
+    ASSERT_THROWS_WITH(
+        torch::tensor(c10::complex<float>(0.1, 0.2), torch::kFloat),
+        "value cannot be converted to type float without overflow");
   }
   {
-    ASSERT_THROWS_WITH(torch::tensor({c10::complex<float>(0.1, 0.2), c10::complex<float>(0.3, 0.4)}, torch::kFloat),
-      "value cannot be converted to type float without overflow");
+    ASSERT_THROWS_WITH(
+        torch::tensor(
+            {c10::complex<float>(0.1, 0.2), c10::complex<float>(0.3, 0.4)},
+            torch::kFloat),
+        "value cannot be converted to type float without overflow");
   }
   {
-    ASSERT_THROWS_WITH(torch::tensor(std::vector<c10::complex<float>>{c10::complex<float>(0.1, 0.2), c10::complex<float>(0.3, 0.4)}, torch::kFloat),
-      "can not do torch::tensor(complex, dtype=non-complex) because complex can not be casted to real number without loss of information");
+    ASSERT_THROWS_WITH(
+        torch::tensor(
+            std::vector<c10::complex<float>>{
+                c10::complex<float>(0.1, 0.2), c10::complex<float>(0.3, 0.4)},
+            torch::kFloat),
+        "can not do torch::tensor(complex, dtype=non-complex) because complex can not be casted to real number without loss of information");
   }
 }
 
-void test_TorchTensorCtorMultiDim_CUDA_expected_dtype(c10::ScalarType default_dtype) {
+void test_TorchTensorCtorMultiDim_CUDA_expected_dtype(
+    c10::ScalarType default_dtype) {
   AutoDefaultDtypeMode dtype_mode(default_dtype);
 
   auto tensor = torch::tensor(
-    {{{{{{{{1.0, 2.0, 3.0}}}}}, {{{{{4.0, 5.0, 6.0}}}}}, {{{{{7.0, 8.0, 9.0}}}}}}}},
-    torch::dtype(default_dtype).device(torch::kCUDA));
+      {{{{{{{{1.0, 2.0, 3.0}}}}},
+         {{{{{4.0, 5.0, 6.0}}}}},
+         {{{{{7.0, 8.0, 9.0}}}}}}}},
+      torch::dtype(default_dtype).device(torch::kCUDA));
   ASSERT_TRUE(tensor.device().is_cuda());
   ASSERT_EQ(tensor.dtype(), default_dtype);
   ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 1, 3, 1, 1, 1, 1, 3}));
   ASSERT_TRUE(torch::allclose(
-    tensor,
-    torch::arange(1, 10, default_dtype).view(tensor.sizes()).to(torch::kCUDA)));
+      tensor,
+      torch::arange(1, 10, default_dtype)
+          .view(tensor.sizes())
+          .to(torch::kCUDA)));
   ASSERT_FALSE(tensor.requires_grad());
 }
 
 TEST(TensorTest, TorchTensorCtorMultiDim_CUDA) {
-  test_TorchTensorCtorMultiDim_CUDA_expected_dtype(/*default_dtype=*/torch::kFloat);
-  test_TorchTensorCtorMultiDim_CUDA_expected_dtype(/*default_dtype=*/torch::kDouble);
+  test_TorchTensorCtorMultiDim_CUDA_expected_dtype(
+      /*default_dtype=*/torch::kFloat);
+  test_TorchTensorCtorMultiDim_CUDA_expected_dtype(
+      /*default_dtype=*/torch::kDouble);
 }
 
-void test_TorchTensorCtorZeroSizedDim_expected_dtype(c10::ScalarType default_dtype) {
+void test_TorchTensorCtorZeroSizedDim_expected_dtype(
+    c10::ScalarType default_dtype) {
   AutoDefaultDtypeMode dtype_mode(default_dtype);
   {
     auto tensor = torch::tensor({});
@@ -729,55 +814,94 @@ void test_TorchTensorCtorZeroSizedDim_expected_dtype(c10::ScalarType default_dty
   {
     auto tensor = torch::tensor({{{{{{{{{{}}}}}}}}}});
     ASSERT_EQ(tensor.numel(), 0);
-    ASSERT_EQ(tensor.sizes(), std::vector<int64_t>({1, 1, 1, 1, 1, 1, 1, 1, 1, 0}));
+    ASSERT_EQ(
+        tensor.sizes(), std::vector<int64_t>({1, 1, 1, 1, 1, 1, 1, 1, 1, 0}));
     ASSERT_EQ(tensor.dtype(), default_dtype);
     ASSERT_FALSE(tensor.requires_grad());
   }
 }
 
 TEST(TensorTest, TorchTensorCtorZeroSizedDim) {
-  test_TorchTensorCtorZeroSizedDim_expected_dtype(/*default_dtype=*/torch::kFloat);
-  test_TorchTensorCtorZeroSizedDim_expected_dtype(/*default_dtype=*/torch::kDouble);
+  test_TorchTensorCtorZeroSizedDim_expected_dtype(
+      /*default_dtype=*/torch::kFloat);
+  test_TorchTensorCtorZeroSizedDim_expected_dtype(
+      /*default_dtype=*/torch::kDouble);
 }
 
-void test_TorchTensorCtorWithoutSpecifyingDtype_expected_dtype(c10::ScalarType default_dtype) {
+void test_TorchTensorCtorWithoutSpecifyingDtype_expected_dtype(
+    c10::ScalarType default_dtype) {
   AutoDefaultDtypeMode dtype_mode(default_dtype);
 
   ASSERT_EQ(torch::tensor({1., 2., 3.}).dtype(), default_dtype);
   ASSERT_EQ(torch::tensor({{1., 2., 3.}}).dtype(), default_dtype);
-  ASSERT_EQ(torch::tensor({1., 2., 3.}, torch::TensorOptions()).dtype(), default_dtype);
-  ASSERT_EQ(torch::tensor({{1., 2., 3.}}, torch::TensorOptions()).dtype(), default_dtype);
+  ASSERT_EQ(
+      torch::tensor({1., 2., 3.}, torch::TensorOptions()).dtype(),
+      default_dtype);
+  ASSERT_EQ(
+      torch::tensor({{1., 2., 3.}}, torch::TensorOptions()).dtype(),
+      default_dtype);
 }
 
 TEST(TensorTest, TorchTensorCtorWithoutSpecifyingDtype) {
   ASSERT_EQ(torch::tensor({1, 2, 3}).dtype(), torch::kLong);
   ASSERT_EQ(torch::tensor({{1, 2, 3}}).dtype(), torch::kLong);
-  ASSERT_EQ(torch::tensor({1, 2, 3}, torch::TensorOptions()).dtype(), torch::kLong);
-  ASSERT_EQ(torch::tensor({{1, 2, 3}}, torch::TensorOptions()).dtype(), torch::kLong);
+  ASSERT_EQ(
+      torch::tensor({1, 2, 3}, torch::TensorOptions()).dtype(), torch::kLong);
+  ASSERT_EQ(
+      torch::tensor({{1, 2, 3}}, torch::TensorOptions()).dtype(), torch::kLong);
 
-  test_TorchTensorCtorWithoutSpecifyingDtype_expected_dtype(/*default_dtype=*/torch::kFloat);
-  test_TorchTensorCtorWithoutSpecifyingDtype_expected_dtype(/*default_dtype=*/torch::kDouble);
+  test_TorchTensorCtorWithoutSpecifyingDtype_expected_dtype(
+      /*default_dtype=*/torch::kFloat);
+  test_TorchTensorCtorWithoutSpecifyingDtype_expected_dtype(
+      /*default_dtype=*/torch::kDouble);
 }
 
-void test_TorchTensorCtorWithNonDtypeOptions_expected_dtype(c10::ScalarType default_dtype) {
+void test_TorchTensorCtorWithNonDtypeOptions_expected_dtype(
+    c10::ScalarType default_dtype) {
   AutoDefaultDtypeMode dtype_mode(default_dtype);
 
-  ASSERT_EQ(torch::tensor({1, 2, 3}, torch::TensorOptions()).dtype(), torch::kLong);
-  ASSERT_EQ(torch::tensor(at::ArrayRef<int>({1, 2, 3}), torch::TensorOptions()).dtype(), torch::kLong);
-  ASSERT_EQ(torch::tensor(std::vector<int>({1, 2, 3}), torch::TensorOptions()).dtype(), torch::kLong);
+  ASSERT_EQ(
+      torch::tensor({1, 2, 3}, torch::TensorOptions()).dtype(), torch::kLong);
+  ASSERT_EQ(
+      torch::tensor(at::ArrayRef<int>({1, 2, 3}), torch::TensorOptions())
+          .dtype(),
+      torch::kLong);
+  ASSERT_EQ(
+      torch::tensor(std::vector<int>({1, 2, 3}), torch::TensorOptions())
+          .dtype(),
+      torch::kLong);
 
-  ASSERT_EQ(torch::tensor({1., 2., 3.}, torch::TensorOptions()).dtype(), default_dtype);
-  ASSERT_EQ(torch::tensor(at::ArrayRef<double>({1., 2., 3.}), torch::TensorOptions()).dtype(), default_dtype);
-  ASSERT_EQ(torch::tensor(std::vector<double>({1., 2., 3.}), torch::TensorOptions()).dtype(), default_dtype);
+  ASSERT_EQ(
+      torch::tensor({1., 2., 3.}, torch::TensorOptions()).dtype(),
+      default_dtype);
+  ASSERT_EQ(
+      torch::tensor(at::ArrayRef<double>({1., 2., 3.}), torch::TensorOptions())
+          .dtype(),
+      default_dtype);
+  ASSERT_EQ(
+      torch::tensor(std::vector<double>({1., 2., 3.}), torch::TensorOptions())
+          .dtype(),
+      default_dtype);
 
-  ASSERT_EQ(torch::tensor({1.f, 2.f, 3.f}, torch::TensorOptions()).dtype(), default_dtype);
-  ASSERT_EQ(torch::tensor(at::ArrayRef<float>({1.f, 2.f, 3.f}), torch::TensorOptions()).dtype(), default_dtype);
-  ASSERT_EQ(torch::tensor(std::vector<float>({1.f, 2.f, 3.f}), torch::TensorOptions()).dtype(), default_dtype);
+  ASSERT_EQ(
+      torch::tensor({1.f, 2.f, 3.f}, torch::TensorOptions()).dtype(),
+      default_dtype);
+  ASSERT_EQ(
+      torch::tensor(
+          at::ArrayRef<float>({1.f, 2.f, 3.f}), torch::TensorOptions())
+          .dtype(),
+      default_dtype);
+  ASSERT_EQ(
+      torch::tensor(std::vector<float>({1.f, 2.f, 3.f}), torch::TensorOptions())
+          .dtype(),
+      default_dtype);
 }
 
 TEST(TensorTest, TorchTensorCtorWithNonDtypeOptions) {
-  test_TorchTensorCtorWithNonDtypeOptions_expected_dtype(/*default_dtype=*/torch::kFloat);
-  test_TorchTensorCtorWithNonDtypeOptions_expected_dtype(/*default_dtype=*/torch::kDouble);
+  test_TorchTensorCtorWithNonDtypeOptions_expected_dtype(
+      /*default_dtype=*/torch::kFloat);
+  test_TorchTensorCtorWithNonDtypeOptions_expected_dtype(
+      /*default_dtype=*/torch::kDouble);
 }
 
 void test_Arange_expected_dtype(c10::ScalarType default_dtype) {
@@ -797,76 +921,80 @@ TEST(TensorTest, Arange) {
 
 TEST(TensorTest, PrettyPrintTensorDataContainer) {
   {
-    ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer(1.1)),
-      "1.1");
+    ASSERT_EQ(c10::str(torch::detail::TensorDataContainer(1.1)), "1.1");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer({1.1, 2.2})),
-      "{1.1, 2.2}");
+        c10::str(torch::detail::TensorDataContainer({1.1, 2.2})), "{1.1, 2.2}");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer({{1, 2}, {3, 4}})),
-      "{{1, 2}, {3, 4}}");
+        c10::str(torch::detail::TensorDataContainer({{1, 2}, {3, 4}})),
+        "{{1, 2}, {3, 4}}");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer({{{{{{{{1.1, 2.2, 3.3}}}}}, {{{{{4.4, 5.5, 6.6}}}}}, {{{{{7.7, 8.8, 9.9}}}}}}}})),
-      "{{{{{{{{1.1, 2.2, 3.3}}}}}, {{{{{4.4, 5.5, 6.6}}}}}, {{{{{7.7, 8.8, 9.9}}}}}}}}");
+        c10::str(torch::detail::TensorDataContainer(
+            {{{{{{{{1.1, 2.2, 3.3}}}}},
+               {{{{{4.4, 5.5, 6.6}}}}},
+               {{{{{7.7, 8.8, 9.9}}}}}}}})),
+        "{{{{{{{{1.1, 2.2, 3.3}}}}}, {{{{{4.4, 5.5, 6.6}}}}}, {{{{{7.7, 8.8, 9.9}}}}}}}}");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer({{{{{{{{{{1}}}}}}}}}})),
-      "{{{{{{{{{{1}}}}}}}}}}");
+        c10::str(torch::detail::TensorDataContainer({{{{{{{{{{1}}}}}}}}}})),
+        "{{{{{{{{{{1}}}}}}}}}}");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer({{{{{{{{{{}}}}}}}}}})),
-      "{{{{{{{{{{}}}}}}}}}}");
+        c10::str(torch::detail::TensorDataContainer({{{{{{{{{{}}}}}}}}}})),
+        "{{{{{{{{{{}}}}}}}}}}");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer({{{{{{{{{{1, 2}}}}}}}}}})),
-      "{{{{{{{{{{1, 2}}}}}}}}}}");
+        c10::str(torch::detail::TensorDataContainer({{{{{{{{{{1, 2}}}}}}}}}})),
+        "{{{{{{{{{{1, 2}}}}}}}}}}");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer(at::ArrayRef<double>({1.1, 2.2}))),
-      "{1.1, 2.2}");
+        c10::str(torch::detail::TensorDataContainer(
+            at::ArrayRef<double>({1.1, 2.2}))),
+        "{1.1, 2.2}");
   }
   {
     ASSERT_EQ(
-      c10::str(torch::detail::TensorDataContainer(std::vector<double>({1.1, 2.2}))),
-      "{1.1, 2.2}");
+        c10::str(torch::detail::TensorDataContainer(
+            std::vector<double>({1.1, 2.2}))),
+        "{1.1, 2.2}");
   }
 }
 
 TEST(TensorTest, TensorDataContainerCallingAccessorOfWrongType) {
   {
     ASSERT_THROWS_WITH(
-      torch::detail::TensorDataContainer(1.1).init_list(),
-      "Can only call `init_list()` on a TensorDataContainer that has `is_init_list() == true`");
+        torch::detail::TensorDataContainer(1.1).init_list(),
+        "Can only call `init_list()` on a TensorDataContainer that has `is_init_list() == true`");
     ASSERT_THROWS_WITH(
-      torch::detail::TensorDataContainer(1.1).tensor(),
-      "Can only call `tensor()` on a TensorDataContainer that has `is_tensor() == true`");
+        torch::detail::TensorDataContainer(1.1).tensor(),
+        "Can only call `tensor()` on a TensorDataContainer that has `is_tensor() == true`");
   }
   {
     ASSERT_THROWS_WITH(
-      torch::detail::TensorDataContainer({1.1, 2.2}).scalar(),
-      "Can only call `scalar()` on a TensorDataContainer that has `is_scalar() == true`");
+        torch::detail::TensorDataContainer({1.1, 2.2}).scalar(),
+        "Can only call `scalar()` on a TensorDataContainer that has `is_scalar() == true`");
     ASSERT_THROWS_WITH(
-      torch::detail::TensorDataContainer({1.1, 2.2}).tensor(),
-      "Can only call `tensor()` on a TensorDataContainer that has `is_tensor() == true`");
+        torch::detail::TensorDataContainer({1.1, 2.2}).tensor(),
+        "Can only call `tensor()` on a TensorDataContainer that has `is_tensor() == true`");
   }
   {
     ASSERT_THROWS_WITH(
-      torch::detail::TensorDataContainer(at::ArrayRef<double>({1.1, 2.2})).scalar(),
-      "Can only call `scalar()` on a TensorDataContainer that has `is_scalar() == true`");
+        torch::detail::TensorDataContainer(at::ArrayRef<double>({1.1, 2.2}))
+            .scalar(),
+        "Can only call `scalar()` on a TensorDataContainer that has `is_scalar() == true`");
     ASSERT_THROWS_WITH(
-      torch::detail::TensorDataContainer(at::ArrayRef<double>({1.1, 2.2})).init_list(),
-      "Can only call `init_list()` on a TensorDataContainer that has `is_init_list() == true`");
+        torch::detail::TensorDataContainer(at::ArrayRef<double>({1.1, 2.2}))
+            .init_list(),
+        "Can only call `init_list()` on a TensorDataContainer that has `is_init_list() == true`");
   }
 }
 
@@ -914,8 +1042,8 @@ TEST(TensorTest, FromBlobWithStrides) {
   ASSERT_EQ(tensor.numel(), 9);
   const std::vector<int64_t> expected_strides = {1, 3};
   ASSERT_EQ(tensor.strides(), expected_strides);
-  for (int64_t i = 0; i < tensor.size(0); ++i) {
-    for (int64_t j = 0; j < tensor.size(1); ++j) {
+  for (const auto i : c10::irange(tensor.size(0))) {
+    for (const auto j : c10::irange(tensor.size(1))) {
       // NOTE: This is column major because the strides are swapped.
       EXPECT_EQ(tensor[i][j].item<int32_t>(), 1 + (j * tensor.size(1)) + i);
     }
@@ -968,17 +1096,24 @@ TEST(TensorTest, BackwardAndGrad) {
 }
 
 TEST(TensorTest, BackwardCreatesOnesGrad) {
-  const auto x = torch::tensor({5}, torch::dtype(torch::kFloat).requires_grad(true));
+  const auto x =
+      torch::tensor({5}, torch::dtype(torch::kFloat).requires_grad(true));
   x.backward();
-  ASSERT_TRUE(torch::equal(x.grad(),
-              torch::ones_like(x)));
+  ASSERT_TRUE(torch::equal(x.grad(), torch::ones_like(x)));
 }
 
 TEST(TensorTest, BackwardNonScalarOutputs) {
   auto x = torch::randn({5, 5}, torch::requires_grad());
   auto y = x * x;
-  ASSERT_THROWS_WITH(y.backward(),
-    "grad can be implicitly created only for scalar outputs");
+  ASSERT_THROWS_WITH(
+      y.backward(), "grad can be implicitly created only for scalar outputs");
+}
+
+TEST(TensorTest, BackwardComplexScalarOutput) {
+  auto x = torch::randn({5, 5}, torch::requires_grad());
+  auto y = (x * c10::Scalar(c10::complex<float>(0, 0.5))).sum();
+  ASSERT_THROWS_WITH(
+      y.backward(), "grad can be computed only for real scalar outputs");
 }
 
 TEST(TensorTest, IsLeaf) {
@@ -1040,15 +1175,18 @@ TEST(TensorTest, RequiresGradInplace) {
   ASSERT_TRUE(x.requires_grad());
 
   auto y = x * x;
-  ASSERT_THROWS_WITH(y.requires_grad_(false),
-    "you can only change requires_grad flags of leaf variables.");
+  ASSERT_THROWS_WITH(
+      y.requires_grad_(false),
+      "you can only change requires_grad flags of leaf variables.");
 
   x.requires_grad_(false);
   ASSERT_FALSE(x.requires_grad());
 
-  const auto int_tensor = torch::tensor({5}, at::TensorOptions().dtype(torch::kInt));
-  ASSERT_THROWS_WITH(int_tensor.requires_grad_(true),
-    "Only Tensors of floating point and complex dtype can require gradients");
+  const auto int_tensor =
+      torch::tensor({5}, at::TensorOptions().dtype(torch::kInt));
+  ASSERT_THROWS_WITH(
+      int_tensor.requires_grad_(true),
+      "Only Tensors of floating point and complex dtype can require gradients");
 }
 
 TEST(TensorTest, StdDimension) {
@@ -1072,3 +1210,61 @@ TEST(TensorTest, StdDimension) {
   ASSERT_EQ(torch::std(x, 0, /*unbiased=*/true).numel(), 3);
   ASSERT_EQ(std::get<0>(torch::std_mean(x, 0, /*unbiased=*/true)).numel(), 3);
 }
+
+TEST(TensorTest, ReshapeAlias) {
+  // Tests the behavior of the _reshape_alias private operator so
+  // that it matches the behavior of as_strided and view.
+  auto x = torch::randn({3, 3});
+  ASSERT_TRUE(torch::equal(
+      torch::_reshape_alias(x, {2, 2}, {1, 2}),
+      torch::as_strided(x, {2, 2}, {1, 2})));
+  ASSERT_TRUE(torch::equal(torch::_reshape_alias(x, {9}, {1}), x.view({-1})));
+
+  // Test that the backward works fine.
+  auto y = torch::randn({3, 3}, torch::requires_grad(true));
+  auto z = torch::clone(y).detach().requires_grad_(true);
+  (y * y).view({-1}).mean().backward();
+  torch::_reshape_alias((z * z), {9}, {1}).mean().backward();
+  ASSERT_TRUE(torch::equal(y.grad(), z.grad()));
+}
+
+TEST(TensorTest, BackendMetadata) {
+  // Tests ability to assign custom backend metadata to tensor.
+
+  struct CustomBackendMetadata : public c10::BackendMeta {
+    mutable bool cloned_{false}; // for testing this field will mutate when
+                                 // clone() is called by shallow_copy_from.
+    c10::intrusive_ptr<c10::BackendMeta> clone(
+        const c10::intrusive_ptr<c10::BackendMeta>& ptr) const override {
+      cloned_ = true;
+      return c10::BackendMeta::clone(ptr);
+    }
+  };
+
+  at::Tensor y;
+  c10::intrusive_ptr<c10::BackendMeta> tmeta{};
+  CustomBackendMetadata* custom_tmeta{nullptr};
+
+  {
+    auto x = torch::ones({3, 3});
+    auto impl{x.unsafeGetTensorImpl()};
+    ASSERT_TRUE(impl != nullptr);
+
+    tmeta = impl->get_backend_meta_intrusive_ptr();
+    ASSERT_TRUE(tmeta == nullptr);
+    c10::intrusive_ptr<c10::BackendMeta> new_tmeta{
+        std::unique_ptr<c10::BackendMeta>(new CustomBackendMetadata())};
+    impl->set_backend_meta(new_tmeta);
+    tmeta = impl->get_backend_meta_intrusive_ptr();
+    ASSERT_TRUE(tmeta == new_tmeta);
+    custom_tmeta = dynamic_cast<CustomBackendMetadata*>(tmeta.get());
+    ASSERT_TRUE(custom_tmeta != nullptr);
+    ASSERT_TRUE(custom_tmeta->cloned_ == false);
+    y.unsafeGetTensorImpl()->shallow_copy_from(x.getIntrusivePtr());
+  }
+
+  ASSERT_TRUE(
+      tmeta == y.unsafeGetTensorImpl()->get_backend_meta_intrusive_ptr());
+  ASSERT_TRUE(tmeta.get() == y.unsafeGetTensorImpl()->get_backend_meta());
+  ASSERT_TRUE(custom_tmeta->cloned_ == true);
+}
