@@ -13,6 +13,9 @@
 #include "torch/csrc/jit/passes/utils/subgraph_utils.h"
 #include "torch/csrc/jit/runtime/argument_spec.h"
 #include "torch/csrc/jit/runtime/autodiff.h"
+#include "torch/csrc/jit/runtime/graph_iterator.h"
+#include "torch/csrc/jit/runtime/profiling_graph_executor_impl.h"
+#include "torch/torch.h"
 
 #include <ATen/ATen.h>
 #include "torch/csrc/autograd/engine.h"
@@ -31,7 +34,9 @@ using test_fn_type = std::function<variable_list(const variable_list&)>;
 struct ADTestSpec {
   ADTestSpec(
       const char* name,
+      // NOLINTNEXTLINE(modernize-pass-by-value)
       var_meta_list input_meta,
+      // NOLINTNEXTLINE(modernize-pass-by-value)
       test_fn_type test_fn,
       float clampMax = -1.0f)
       : name(name),
@@ -116,14 +121,10 @@ TEST(AutodiffTest, ADFormulas) {
       {"t", unary_pointwise_2d, [](const VL& v) -> VL { return {v[0].t()}; }},
       {"view",
        unary_pointwise_2d,
-       [](const VL& v) -> VL {
-         return {v[0].view({3, 2})};
-       }},
+       [](const VL& v) -> VL { return {v[0].view({3, 2})}; }},
       {"expand",
        {{2, 1}},
-       [](const VL& v) -> VL {
-         return {v[0].expand({2, 3})};
-       }},
+       [](const VL& v) -> VL { return {v[0].expand({2, 3})}; }},
       {"mm",
        {{10, 12}, {12, 15}},
        [](const VL& v) -> VL { return {v[0].mm(v[1])}; }},
@@ -164,8 +165,7 @@ TEST(AutodiffTest, ADFormulas) {
     // Get outputs from the interpreter
     auto tensors_in = fmap(vars_in, cast);
     auto tensor_grads_in = fmap(var_grads_in, cast);
-    tensor_list tensors_out, tensor_grads_out;
-    std::tie(tensors_out, tensor_grads_out) =
+    auto [tensors_out, tensor_grads_out] =
         runGradient(grad_spec, tensors_in, tensor_grads_in);
 
     // Compare results
@@ -279,5 +279,65 @@ TEST(AutodiffTest, DifferentiateWithRequiresGrad) {
       ->run(*grad_spec.df);
 }
 
+class AutodiffRemoveUnusedGradientsTest : public ::testing::Test {
+ protected:
+  void SetUp() override {
+    prev_exec = getExecutorMode();
+    getExecutorMode() = true;
+    prev_inline_autodiff = getAutodiffSubgraphInlining();
+    debugSetAutodiffSubgraphInlining(false);
+  }
+  void TearDown() override {
+    getExecutorMode() = prev_exec;
+    debugSetAutodiffSubgraphInlining(prev_inline_autodiff);
+  }
+
+  bool prev_exec;
+  bool prev_profiling;
+  bool prev_inline_autodiff;
+};
+
+TEST_F(AutodiffRemoveUnusedGradientsTest, Linear) {
+  auto graph = std::make_shared<Graph>();
+  const std::string input =
+      R"IR(
+graph(%inp.1 : Tensor,
+      %weight.1 : Tensor,
+      %bias.1 : Tensor):
+  %6 : Tensor = aten::linear(%inp.1, %weight.1, %bias.1)
+  return (%6))IR";
+  parseIR(input, graph.get());
+
+  auto inp = torch::randn({10, 10}).requires_grad_(false);
+  auto weight = torch::randn({10, 10}).requires_grad_(true);
+  auto bias = torch::randn({1, 10}).requires_grad_(true);
+  auto stack = createStack({inp, weight, bias});
+
+  ProfilingGraphExecutorImpl executor(graph, "linear");
+
+  // initial run to profile requires_grad information
+  auto plan = executor.getPlanFor(stack, 20);
+  InterpreterState is{plan.code};
+  is.run(stack);
+
+  auto optimized_plan = executor.getPlanFor(stack, 20);
+  DepthFirstGraphNodeIterator it(optimized_plan.graph);
+  Node* diff_graph_node = nullptr;
+
+  while ((diff_graph_node = it.next()) != nullptr) {
+    if (diff_graph_node->kind() == prim::DifferentiableGraph) {
+      break;
+    }
+  }
+  ASSERT_NE(nullptr, diff_graph_node);
+
+  auto backward_graph = diff_graph_node->g(attr::ReverseSubgraph);
+
+  // we expect to compute grad_weight (which requires a matmul) but we don't
+  // expect to compute grad_input. So, we expect exactly 1 matmul.
+  // Note: this could change, e.g. if mm is used instead
+  testing::FileCheck().check_count("matmul", 1, true)->run(*backward_graph);
+}
+
 } // namespace jit
 } // namespace torch
