@@ -1,11 +1,19 @@
+# Owner(s): ["module: unknown"]
+
 import os.path
+import sys
 import tempfile
+import unittest
+
+from model import get_custom_op_library_path, Model
 
 import torch
+import torch._library.utils as utils
 from torch import ops
+from torch.testing._internal.common_utils import IS_WINDOWS, run_tests, TestCase
 
-from model import Model, get_custom_op_library_path
-from torch.testing._internal.common_utils import TestCase, run_tests
+
+torch.ops.import_module("pointwise")
 
 
 class TestCustomOperators(TestCase):
@@ -16,6 +24,64 @@ class TestCustomOperators(TestCase):
     def test_custom_library_is_loaded(self):
         self.assertIn(self.library_path, ops.loaded_libraries)
 
+    def test_op_with_no_abstract_impl_pystub(self):
+        x = torch.randn(3, device="meta")
+        if utils.requires_set_python_module():
+            with self.assertRaisesRegex(RuntimeError, "pointwise"):
+                torch.ops.custom.tan(x)
+        else:
+            # Smoketest
+            torch.ops.custom.tan(x)
+
+    def test_op_with_incorrect_abstract_impl_pystub(self):
+        x = torch.randn(3, device="meta")
+        with self.assertRaisesRegex(RuntimeError, "pointwise"):
+            torch.ops.custom.cos(x)
+
+    @unittest.skipIf(IS_WINDOWS, "torch.compile not supported on windows")
+    def test_dynamo_pystub_suggestion(self):
+        x = torch.randn(3)
+
+        @torch.compile(backend="eager", fullgraph=True)
+        def f(x):
+            return torch.ops.custom.asin(x)
+
+        with self.assertRaisesRegex(
+            RuntimeError,
+            r"(?s)Operator does not support running with fake tensors.*you may need to `import nonexistent`",
+        ):
+            f(x)
+
+    def test_abstract_impl_pystub_faketensor(self):
+        from functorch import make_fx
+
+        x = torch.randn(3, device="cpu")
+        self.assertNotIn("my_custom_ops", sys.modules.keys())
+
+        with self.assertRaises(
+            torch._subclasses.fake_tensor.UnsupportedOperatorException
+        ):
+            gm = make_fx(torch.ops.custom.nonzero.default, tracing_mode="symbolic")(x)
+
+        torch.ops.import_module("my_custom_ops")
+        gm = make_fx(torch.ops.custom.nonzero.default, tracing_mode="symbolic")(x)
+        self.assertExpectedInline(
+            """\
+def forward(self, arg0_1):
+    nonzero = torch.ops.custom.nonzero.default(arg0_1);  arg0_1 = None
+    return nonzero
+""".strip(),
+            gm.code.strip(),
+        )
+
+    def test_abstract_impl_pystub_meta(self):
+        x = torch.randn(3, device="meta")
+        self.assertNotIn("my_custom_ops2", sys.modules.keys())
+        with self.assertRaisesRegex(NotImplementedError, r"'my_custom_ops2'"):
+            torch.ops.custom.sin.default(x)
+        torch.ops.import_module("my_custom_ops2")
+        torch.ops.custom.sin.default(x)
+
     def test_calling_custom_op_string(self):
         output = ops.custom.op2("abc", "def")
         self.assertLess(output, 0)
@@ -44,8 +110,8 @@ class TestCustomOperators(TestCase):
         output.sum().backward(go, False, True)
         grad = torch.ones(5, 5)
 
-        self.assertTrue(torch.allclose(x.grad, y + grad))
-        self.assertTrue(torch.allclose(y.grad, x + grad * 2))
+        self.assertEqual(x.grad, y + grad)
+        self.assertEqual(y.grad, x + grad * 2)
 
         # Test with optional arg.
         x.grad.zero_()
@@ -56,9 +122,9 @@ class TestCustomOperators(TestCase):
 
         go = torch.ones((), requires_grad=True)
         output.sum().backward(go, False, True)
-        self.assertTrue(torch.allclose(x.grad, y + grad))
-        self.assertTrue(torch.allclose(y.grad, x + grad * 2))
-        self.assertTrue(torch.allclose(z.grad, grad))
+        self.assertEqual(x.grad, y + grad)
+        self.assertEqual(y.grad, x + grad * 2)
+        self.assertEqual(z.grad, grad)
 
     def test_calling_custom_op_with_autograd_in_nograd_mode(self):
         with torch.no_grad():
