@@ -2,14 +2,16 @@
 
 import json
 import os
-import pathlib
 import sys
 import tempfile
 from unittest.mock import MagicMock
 
 import torch
 from torch.distributed.checkpoint import DefaultLoadPlanner
-from torch.distributed.checkpoint._hf_utils import _HFStorageInfo
+from torch.distributed.checkpoint._hf_utils import (
+    _HFStorageInfo,
+    NUM_BYTES_FOR_HEADER_LEN,
+)
 from torch.distributed.checkpoint.default_planner import DefaultSavePlanner
 from torch.distributed.checkpoint.filesystem import _StorageInfo, FileSystem
 from torch.distributed.checkpoint.hf_storage import (
@@ -106,7 +108,7 @@ class TestHfStorage(TestCase):
         with tempfile.TemporaryDirectory() as path:
             writer = HuggingFaceStorageWriter(
                 path=path,
-                save_sharded=True,
+                save_distributed=True,
             )
             writer.fs = FileSystem()
 
@@ -160,30 +162,45 @@ class TestHfStorage(TestCase):
             )
 
     def test_read_data_hf(self) -> None:
-        mock_safetensors = MagicMock()
-        sys.modules["safetensors"] = mock_safetensors
-
-        # Create test tensors
         tensor_0 = torch.tensor([1.0, 2.0, 3.0, 4.0])
 
-        # Mock the deserialize function to return our test tensors
-        # The format matches what's expected in the read_data method
-        mock_safetensors.deserialize.return_value = [
-            (
-                "tensor_0",
-                {"data": tensor_0.numpy().tobytes(), "dtype": "F32", "shape": [4]},
-            ),
-        ]
+        mock_safe_open = MagicMock()
+        mock_context = MagicMock()
+        mock_context.__enter__.return_value.get_slice.return_value = tensor_0
+        mock_safe_open.return_value = mock_context
+
+        sys.modules["safetensors"] = MagicMock()
+        sys.modules["safetensors"].safe_open = mock_safe_open
 
         with tempfile.TemporaryDirectory() as path:
             # Create the reader
             reader = HuggingFaceStorageReader(path=path)
-            reader.fs = FileSystem()
 
             # Create test file
             file_name = "model-00001-of-00001.safetensors"
             file_path = os.path.join(path, file_name)
-            pathlib.Path(file_path).touch()
+
+            with open(file_path, "wb") as f:
+                # write metadata the same way it would be in safetensors file
+                metadata_contents = json.dumps(
+                    {
+                        "tensor_0": {
+                            "dtype": "F32",
+                            "shape": [1, 4],
+                            "data_offsets": [0, 16],
+                        }
+                    }
+                )
+                metadata_bytes = metadata_contents.encode("utf-8")
+
+                f.write(
+                    len(metadata_bytes).to_bytes(
+                        NUM_BYTES_FOR_HEADER_LEN, byteorder="little"
+                    )
+                )
+                f.write(metadata_bytes)
+
+                f.write(tensor_0.numpy().tobytes())
 
             # Set up storage data with _StorageInfo objects
             storage_data = {
@@ -191,8 +208,6 @@ class TestHfStorage(TestCase):
                     fqn="tensor_0", offset=torch.Size([0]), index=None
                 ): _HFStorageInfo(
                     file_path,
-                    0,
-                    tensor_0.numel() * tensor_0.element_size(),
                     tensor_0.shape,
                     tensor_0.dtype,
                 ),
@@ -245,13 +260,15 @@ class TestHfStorage(TestCase):
                 ),
             )
 
-            # Call read_data
             future = reader.read_data(load_plan, load_planner)
             future.wait()
 
             # Verify results - the target tensors should now contain the values from our test tensor
             self.assertTrue(torch.equal(state_dict["tensor_0"], tensor_0))
 
+            mock_safe_open.assert_called_once_with(filename=file_path, framework="pt")
+            mock_context.__enter__.return_value.get_slice.assert_called_with("tensor_0")
+
     def test_write_metadata_hf(self) -> None:
         mock_module = MagicMock()
         sys.modules["huggingface_hub"] = mock_module
@@ -305,28 +322,50 @@ class TestHfStorage(TestCase):
                 self.assertEqual(metadata, expected_metadata)
 
     def test_read_metadata_hf(self):
+        mock_safe_open = MagicMock()
+        mock_context = MagicMock()
+
+        mock_safe_open.return_value = mock_context
+
+        mock_context.__enter__.return_value.keys.return_value = ["tensor_0"]
+        mock_context.__enter__.return_value.metadata.return_value = {}
+
+        mock_slice = MagicMock()
+        mock_slice.get_shape.return_value = [5, 10]
+        mock_slice.get_dtype.return_value = "F32"
+        mock_context.__enter__.return_value.get_slice.return_value = mock_slice
+
+        mock_safetensors = MagicMock()
+        mock_safetensors.safe_open = mock_safe_open
+
+        mock_safetensors.torch._getdtype = MagicMock(return_value=torch.float32)
+
+        sys.modules["safetensors"] = mock_safetensors
+        sys.modules["safetensors.torch"] = mock_safetensors.torch
+
         with tempfile.TemporaryDirectory() as path:
             reader = HuggingFaceStorageReader(path=path)
 
             key = "tensor_0"
             file_name = "test.safetensors"
-            with open(os.path.join(path, file_name), "wb") as f:
-                # write metadata the same way it would be in safetensors file
-                metadata_contents = json.dumps(
-                    {
-                        "tensor_0": {
-                            "dtype": "F32",
-                            "shape": [5, 10],
-                            "data_offsets": [0, 200],
-                        }
-                    }
-                )
-                metadata_bytes = metadata_contents.encode("utf-8")
+            file_path = os.path.join(path, file_name)
 
-                f.write(len(metadata_bytes).to_bytes(8, byteorder="little"))
-                f.write(metadata_bytes)
+            # Create an empty file so fs.ls can find it
+            with open(file_path, "wb") as _:
+                pass
 
-            metadata = reader.read_metadata()
+            # Mock the fs.ls method to return our test file
+            original_ls = reader.fs.ls
+            reader.fs.ls = MagicMock(return_value=[file_path])
+
+            try:
+                metadata = reader.read_metadata()
+            finally:
+                # Restore the original ls method
+                reader.fs.ls = original_ls
+
+            # Verify that safe_open was called with our file path
+            mock_safe_open.assert_called_once_with(file_path, framework="pt")
 
             self.assertEqual(
                 metadata.state_dict_metadata,
@@ -342,6 +381,7 @@ class TestHfStorage(TestCase):
                     ),
                 },
             )
+
             self.assertEqual(
                 metadata.storage_data,
                 {
@@ -349,8 +389,6 @@ class TestHfStorage(TestCase):
                         fqn=key, offset=torch.Size([0, 0]), index=None
                     ): _HFStorageInfo(
                         os.path.join(path, file_name),
-                        0,
-                        200,
                         torch.Size([5, 10]),
                         torch.float32,
                     )
