@@ -34,7 +34,6 @@ from torch.distributed.elastic.multiprocessing.errors import ProcessFailure
 from torch.distributed.elastic.rendezvous import RendezvousHandler, RendezvousParameters
 from torch.distributed.elastic.rendezvous.api import RendezvousGracefulExitError
 from torch.distributed.elastic.utils.distributed import get_free_port
-from torch.testing._internal.common_utils import run_tests
 
 
 def do_nothing():
@@ -127,9 +126,7 @@ class TestAgent(SimpleElasticAgent):
         self.stop_workers_call_count = 0
         self.start_workers_call_count = 0
 
-    def _stop_workers(
-        self, worker_group: WorkerGroup, is_restart: bool = False
-    ) -> None:
+    def _stop_workers(self, worker_group: WorkerGroup) -> None:
         # workers are fake, nothing to stop; just clear the rdzv info
         worker_group.group_rank = None
         worker_group.group_world_size = None
@@ -168,6 +165,7 @@ class SimpleElasticAgentTest(unittest.TestCase):
         role="test_trainer",
         local_world_size=8,
         local_addr=None,
+        event_log_handler="null",
     ):
         run_id = str(uuid.uuid4().int)
         port = get_free_port()
@@ -194,6 +192,7 @@ class SimpleElasticAgentTest(unittest.TestCase):
             max_restarts=max_restarts,
             monitor_interval=monitor_interval,
             local_addr=local_addr,
+            event_log_handler=event_log_handler,
         )
         return spec
 
@@ -350,7 +349,9 @@ class SimpleElasticAgentTest(unittest.TestCase):
         self.assertEqual(spec_local_addr, worker_group.master_addr)
         self.assertGreater(worker_group.master_port, 0)
 
-    def test_initialize_workers(self):
+    @patch.object(TestAgent, "_construct_event")
+    @patch("torch.distributed.elastic.agent.server.api.record")
+    def test_initialize_workers(self, mock_record, mock_construct_event):
         spec = self._get_worker_spec(max_restarts=1)
         agent = TestAgent(spec)
         worker_group = agent.get_worker_group()
@@ -361,6 +362,33 @@ class SimpleElasticAgentTest(unittest.TestCase):
             worker = worker_group.workers[i]
             self.assertEqual(worker.id, worker.global_rank)
 
+        mock_construct_event.assert_called()
+        self.assertEqual(mock_construct_event.call_count, 10)
+        mock_record.assert_called()
+        second_arg = mock_record.call_args_list[0][0][1]
+        self.assertEqual(second_arg, "null")
+
+    @patch.object(TestAgent, "_construct_event")
+    @patch("torch.distributed.elastic.agent.server.api.record")
+    def test_initialize_workers_with_new_spec(self, mock_record, mock_construct_event):
+        spec = self._get_worker_spec(
+            max_restarts=1, event_log_handler="framework_logger"
+        )
+        agent = TestAgent(spec)
+        worker_group = agent.get_worker_group()
+        agent._initialize_workers(worker_group)
+
+        self.assertEqual(WorkerState.HEALTHY, worker_group.state)
+        for i in range(spec.local_world_size):
+            worker = worker_group.workers[i]
+            self.assertEqual(worker.id, worker.global_rank)
+
+        mock_construct_event.assert_called()
+        self.assertEqual(mock_construct_event.call_count, 10)
+        mock_record.assert_called()
+        second_arg = mock_record.call_args_list[0][0][1]
+        self.assertEqual(second_arg, "framework_logger")
+
     def test_restart_workers(self):
         spec = self._get_worker_spec()
         agent = TestAgent(spec)
@@ -648,4 +676,7 @@ class SimpleElasticAgentTest(unittest.TestCase):
 
 
 if __name__ == "__main__":
-    run_tests()
+    raise RuntimeError(
+        "This test is not currently used and should be "
+        "enabled in discover_tests.py if required."
+    )
