@@ -4,12 +4,15 @@
 # ruff: noqa
 # flake8: noqa
 
+# Test copied from
+# https://raw.githubusercontent.com/python/cpython/refs/tags/v3.13.5/Lib/test/test_int.py
+
 import sys
 import torch
 import torch._dynamo.test_case
 import unittest
 from torch._dynamo.test_case import CPythonTestCase
-from torch.testing._internal.common_utils import run_tests
+from torch.testing._internal.common_utils import run_tests, skipIfTorchDynamo
 
 __TestCase = CPythonTestCase
 
@@ -433,11 +436,13 @@ class IntTestCases(__TestCase):
             int('0', 5.0)
 
     def test_int_base_indexable(self):
-        class MyIndexable(object):
-            def __init__(self, value):
-                self.value = value
-            def __index__(self):
-                return self.value
+        with torch._dynamo.error_on_graph_break(False):
+            with torch._dynamo.error_on_graph_break(False):
+                class MyIndexable(object):
+                    def __init__(self, value):
+                        self.value = value
+                    def __index__(self):
+                        return self.value
 
         # Check out of range bases.
         for base in 2**100, -2**100, 1, 37:
@@ -452,9 +457,11 @@ class IntTestCases(__TestCase):
     def test_non_numeric_input_types(self):
         # Test possible non-numeric types for the argument x, including
         # subclasses of the explicitly documented accepted types.
-        class CustomStr(str): pass
-        class CustomBytes(bytes): pass
-        class CustomByteArray(bytearray): pass
+
+        with torch._dynamo.error_on_graph_break(False):
+            class CustomStr(str): pass
+            class CustomBytes(bytes): pass
+            class CustomByteArray(bytearray): pass
 
         factories = [
             bytes,
@@ -496,72 +503,82 @@ class IntTestCases(__TestCase):
 
     def test_intconversion(self):
         # Test __int__()
-        class ClassicMissingMethods:
-            pass
+        with torch._dynamo.error_on_graph_break(False):
+            class ClassicMissingMethods:
+                pass
         self.assertRaises(TypeError, int, ClassicMissingMethods())
 
-        class MissingMethods(object):
-            pass
+        with torch._dynamo.error_on_graph_break(False):
+            class MissingMethods(object):
+                pass
         self.assertRaises(TypeError, int, MissingMethods())
 
-        class Foo0:
-            def __int__(self):
-                return 42
+        with torch._dynamo.error_on_graph_break(False):
+            class Foo0:
+                def __int__(self):
+                    return 42
 
         self.assertEqual(int(Foo0()), 42)
 
-        class Classic:
-            pass
+        with torch._dynamo.error_on_graph_break(False):
+            class Classic:
+                pass
         for base in (object, Classic):
-            class IntOverridesTrunc(base):
-                def __int__(self):
-                    return 42
-                def __trunc__(self):
-                    return -12
+            with torch._dynamo.error_on_graph_break(False):
+                class IntOverridesTrunc(base):
+                    def __int__(self):
+                        return 42
+                    def __trunc__(self):
+                        return -12
             self.assertEqual(int(IntOverridesTrunc()), 42)
 
-            class JustTrunc(base):
-                def __trunc__(self):
-                    return 42
+            with torch._dynamo.error_on_graph_break(False):
+                class JustTrunc(base):
+                    def __trunc__(self):
+                        return 42
             with self.assertWarns(DeprecationWarning):
                 self.assertEqual(int(JustTrunc()), 42)
 
-            class ExceptionalTrunc(base):
-                def __trunc__(self):
-                    1 / 0
+            with torch._dynamo.error_on_graph_break(False):
+                class ExceptionalTrunc(base):
+                    def __trunc__(self):
+                        1 / 0
             with self.assertRaises(ZeroDivisionError), \
                  self.assertWarns(DeprecationWarning):
                 int(ExceptionalTrunc())
 
             for trunc_result_base in (object, Classic):
-                class Index(trunc_result_base):
-                    def __index__(self):
-                        return 42
+                with torch._dynamo.error_on_graph_break(False):
+                    class Index(trunc_result_base):
+                        def __index__(self):
+                            return 42
 
-                class TruncReturnsNonInt(base):
-                    def __trunc__(self):
-                        return Index()
+                    class TruncReturnsNonInt(base):
+                        def __trunc__(self):
+                            return Index()
                 with self.assertWarns(DeprecationWarning):
                     self.assertEqual(int(TruncReturnsNonInt()), 42)
 
-                class Intable(trunc_result_base):
-                    def __int__(self):
-                        return 42
+                with torch._dynamo.error_on_graph_break(False):
+                    class Intable(trunc_result_base):
+                        def __int__(self):
+                            return 42
 
-                class TruncReturnsNonIndex(base):
-                    def __trunc__(self):
-                        return Intable()
+                    class TruncReturnsNonIndex(base):
+                        def __trunc__(self):
+                            return Intable()
                 with self.assertWarns(DeprecationWarning):
                     self.assertEqual(int(TruncReturnsNonInt()), 42)
 
-                class NonIntegral(trunc_result_base):
-                    def __trunc__(self):
-                        # Check that we avoid infinite recursion.
-                        return NonIntegral()
+                with torch._dynamo.error_on_graph_break(False):
+                    class NonIntegral(trunc_result_base):
+                        def __trunc__(self):
+                            # Check that we avoid infinite recursion.
+                            return NonIntegral()
 
-                class TruncReturnsNonIntegral(base):
-                    def __trunc__(self):
-                        return NonIntegral()
+                    class TruncReturnsNonIntegral(base):
+                        def __trunc__(self):
+                            return NonIntegral()
                 try:
                     with self.assertWarns(DeprecationWarning):
                         int(TruncReturnsNonIntegral())
@@ -573,27 +590,29 @@ class IntTestCases(__TestCase):
                     self.fail("Failed to raise TypeError with %s" %
                               ((base, trunc_result_base),))
 
-                # Regression test for bugs.python.org/issue16060.
-                class BadInt(trunc_result_base):
-                    def __int__(self):
-                        return 42.0
+                with torch._dynamo.error_on_graph_break(False):
+                    # Regression test for bugs.python.org/issue16060.
+                    class BadInt(trunc_result_base):
+                        def __int__(self):
+                            return 42.0
 
-                class TruncReturnsBadInt(base):
-                    def __trunc__(self):
-                        return BadInt()
+                    class TruncReturnsBadInt(base):
+                        def __trunc__(self):
+                            return BadInt()
 
                 with self.assertRaises(TypeError), \
                      self.assertWarns(DeprecationWarning):
                     int(TruncReturnsBadInt())
 
     def test_int_subclass_with_index(self):
-        class MyIndex(int):
-            def __index__(self):
-                return 42
+        with torch._dynamo.error_on_graph_break(False):
+            class MyIndex(int):
+                def __index__(self):
+                    return 42
 
-        class BadIndex(int):
-            def __index__(self):
-                return 42.0
+            class BadIndex(int):
+                def __index__(self):
+                    return 42.0
 
         my_int = MyIndex(7)
         self.assertEqual(my_int, 7)
@@ -602,13 +621,14 @@ class IntTestCases(__TestCase):
         self.assertEqual(int(BadIndex()), 0)
 
     def test_int_subclass_with_int(self):
-        class MyInt(int):
-            def __int__(self):
-                return 42
+        with torch._dynamo.error_on_graph_break(False):
+            class MyInt(int):
+                def __int__(self):
+                    return 42
 
-        class BadInt(int):
-            def __int__(self):
-                return 42.0
+            class BadInt(int):
+                def __int__(self):
+                    return 42.0
 
         my_int = MyInt(7)
         self.assertEqual(my_int, 7)
@@ -619,33 +639,34 @@ class IntTestCases(__TestCase):
         self.assertRaises(TypeError, int, my_int)
 
     def test_int_returns_int_subclass(self):
-        class BadIndex:
-            def __index__(self):
-                return True
+        with torch._dynamo.error_on_graph_break(False):
+            class BadIndex:
+                def __index__(self):
+                    return True
 
-        class BadIndex2(int):
-            def __index__(self):
-                return True
+            class BadIndex2(int):
+                def __index__(self):
+                    return True
 
-        class BadInt:
-            def __int__(self):
-                return True
+            class BadInt:
+                def __int__(self):
+                    return True
 
-        class BadInt2(int):
-            def __int__(self):
-                return True
+            class BadInt2(int):
+                def __int__(self):
+                    return True
 
-        class TruncReturnsBadIndex:
-            def __trunc__(self):
-                return BadIndex()
+            class TruncReturnsBadIndex:
+                def __trunc__(self):
+                    return BadIndex()
 
-        class TruncReturnsBadInt:
-            def __trunc__(self):
-                return BadInt()
+            class TruncReturnsBadInt:
+                def __trunc__(self):
+                    return BadInt()
 
-        class TruncReturnsIntSubclass:
-            def __trunc__(self):
-                return True
+            class TruncReturnsIntSubclass:
+                def __trunc__(self):
+                    return True
 
         bad_int = BadIndex()
         with self.assertWarns(DeprecationWarning):
@@ -690,6 +711,7 @@ class IntTestCases(__TestCase):
         self.assertEqual(n, 1)
         self.assertIs(type(n), IntSubclass)
 
+    @skipIfTorchDynamo("flaky under dynamo")
     def test_error_message(self):
         def check(s, base=None):
             with self.assertRaises(ValueError,
