@@ -248,9 +248,9 @@ class _ReferenceConvBnNd(torch.nn.Conv2d, torch.nn.modules.conv._ConvNd):
             + cls._FLOAT_MODULE.__name__
         )
         if not qconfig:
-            assert hasattr(
-                mod, "qconfig"
-            ), "Input float module must have qconfig defined"
+            assert hasattr(mod, "qconfig"), (
+                "Input float module must have qconfig defined"
+            )
             assert mod.qconfig, "Input float module must have a valid qconfig"
             qconfig = mod.qconfig
         conv, bn = mod[0], mod[1]
