@@ -99,17 +99,17 @@ class OnDevicePTQUtils:
         ):
             raise ValueError("Quantized weight must be produced.")
         fp_weight = weight.inputsAt(0).node()
-        assert (
-            fp_weight.kind() == "prim::GetAttr"
-        ), "Weight must be an attribute of the module."
+        assert fp_weight.kind() == "prim::GetAttr", (
+            "Weight must be an attribute of the module."
+        )
         fp_weight_name = fp_weight.s("name")
         return fp_weight_name
 
     @staticmethod
     def is_per_channel_quantized_packed_param(node):
-        assert (
-            node.kind() == "quantized::linear_prepack"
-        ), "Node must corresponds to linear_prepack."
+        assert node.kind() == "quantized::linear_prepack", (
+            "Node must corresponds to linear_prepack."
+        )
         weight = node.inputsAt(0).node()
         assert (
             weight.kind() != "aten::quantize_per_tensor"
