@@ -1,6 +1,10 @@
+# Owner(s): ["module: named tensor"]
+# ruff: noqa: F841
 import unittest
 from torch.testing._internal.common_utils import TestCase, run_tests, TEST_NUMPY
+from torch.testing._internal.common_utils import skipIfTorchDynamo
 from torch.testing._internal.common_cuda import TEST_CUDA
+from torch.testing._internal.common_device_type import get_all_device_types
 from collections import namedtuple, OrderedDict
 import itertools
 import functools
@@ -89,8 +93,7 @@ class TestNamedTensor(TestCase):
             return
         result = op(*args)
         self.assertEqual(result.names, expected_names,
-                         msg='Name inference for {} on device {} failed'.format(
-                             op.__name__, device))
+                         msg=f'Name inference for {op.__name__} on device {device} failed')
 
     # TODO(rzou): Some form of this check should be added to self.assertEqual.
     # Right now I don't know what it should look like.
@@ -146,6 +149,7 @@ class TestNamedTensor(TestCase):
             names65 = ['A' * i for i in range(1, 66)]
             x = factory([1] * 65, names=names64, device=device)
 
+    @skipIfTorchDynamo("not a bug: Dynamo causes the refcounts to be different")
     def test_none_names_refcount(self, N=10):
         def scope():
             unnamed = torch.empty(2, 3)
@@ -223,7 +227,7 @@ class TestNamedTensor(TestCase):
             names)
 
     def test_index_fill(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             expected_names = ('N', 'C')
             x = torch.randn(3, 5, device=device, names=expected_names)
 
@@ -240,7 +244,7 @@ class TestNamedTensor(TestCase):
             self.assertEqual(output.names, expected_names)
 
     def test_equal(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             tensor = torch.randn(2, 3, device=device)
             other = tensor.clone()
 
@@ -282,7 +286,7 @@ class TestNamedTensor(TestCase):
             self.assertEqual(values.names, expected_names)
             self.assertEqual(indices.names, expected_names)
 
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
 
             named_tensor_1d = torch.zeros(2, 3, 5, device=device, names=list('ABC'))
             named_tensor_2d = torch.zeros(2, 3, 5, 7, device=device, names=list('ABCD'))
@@ -296,6 +300,15 @@ class TestNamedTensor(TestCase):
             check_tuple_return(F.max_pool2d_with_indices, [named_tensor_2d, [2, 2]], named_tensor_2d.names)
             check_tuple_return(F.max_pool3d_with_indices, [named_tensor_3d, [2, 2, 2]], named_tensor_3d.names)
 
+    def test_max_pooling_without_names_does_not_warn(self):
+        for device in get_all_device_types():
+            tensor_2d = torch.zeros(2, 3, 5, 7, device=device, requires_grad=True)
+            with warnings.catch_warnings(record=True) as warns:
+                warnings.simplefilter("always")
+                result = F.max_pool2d(tensor_2d, [2, 2])
+                result.sum().backward()
+                self.assertEqual(len(warns), 0)
+
     def test_no_save_support(self):
         named_tensor = torch.zeros(2, 3, names=('N', 'C'))
         buf = io.BytesIO()
@@ -316,14 +329,14 @@ class TestNamedTensor(TestCase):
     def test_big_tensor_repr_has_names(self):
         def check_repr(named_tensor):
             unnamed_tensor = named_tensor.rename(None)
-            names_tag = 'names={}'.format(named_tensor.names)
+            names_tag = f'names={named_tensor.names}'
             self.assertIn(names_tag, repr(named_tensor))
 
         check_repr(torch.randn(128, 3, 64, 64, names=('N', 'C', 'H', 'W')))
 
     def test_noncontig_contiguous(self):
         # This type of contiguous is special-cased and therefore needs its own test
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             x = torch.randn(2, 3, device=device).t().rename_('N', 'C')
             self.assertEqual(x.contiguous().names, ('N', 'C'))
 
@@ -440,7 +453,7 @@ class TestNamedTensor(TestCase):
             tensor.names = ['N', 'N']
 
     def test_factory_edge_cases(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             self._test_factory(torch.empty, device)
 
     def test_factory_coverage(self):
@@ -462,11 +475,11 @@ class TestNamedTensor(TestCase):
             torch.zeros,
         ]
 
-        for op, device in itertools.product(supported, torch.testing.get_all_device_types()):
+        for op, device in itertools.product(supported, get_all_device_types()):
             _test(op, device)
 
         # Test torch.full
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             names = ('N', 'T', 'D')
             result = torch.full([1, 2, 3], 2., names=names, device=device)
             expected = torch.full([1, 2, 3], 2., device=device).rename_(*names)
@@ -518,8 +531,6 @@ class TestNamedTensor(TestCase):
         t = torch.empty(2, 3, 5, names=('N', None, 'C'))
         self.assertEqual(t.size('N'), 2)
         self.assertEqual(t.size('C'), 5)
-        with self.assertRaisesRegex(RuntimeError, 'Please look up dimensions by name*'):
-            t.size(None)
         with self.assertRaisesRegex(RuntimeError, 'Name \'channels\' not found in '):
             t.size('channels')
         with self.assertRaisesRegex(RuntimeError, 'Name \'N\' not found in '):
@@ -529,8 +540,6 @@ class TestNamedTensor(TestCase):
         t = torch.empty(2, 3, 5, names=('N', None, 'C'))
         self.assertEqual(t.stride('N'), 3 * 5)
         self.assertEqual(t.stride('C'), 1)
-        with self.assertRaisesRegex(RuntimeError, 'Please look up dimensions by name'):
-            t.stride(None)
         with self.assertRaisesRegex(RuntimeError, 'Name \'channels\' not found in '):
             t.stride('channels')
         with self.assertRaisesRegex(RuntimeError, 'Name \'N\' not found in '):
@@ -545,7 +554,7 @@ class TestNamedTensor(TestCase):
         self.assertEqual(t.t().names, ['C', 'N'])
 
     def test_resize(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             named = torch.randn(2, names=('N',), device=device)
             named.resize_([2])
             self.assertEqual(named.names, ['N'])
@@ -567,7 +576,7 @@ class TestNamedTensor(TestCase):
             self.assertEqual(unnamed.names, ['N'])
 
     def test_cdist(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             tensor = torch.randn(3, 1, 2, 7, names=('M', 'N', 'first_group', 'features'),
                                  device=device)
             other = torch.randn(5, 11, 7, names=('N', 'second_group', 'features'),
@@ -638,7 +647,7 @@ class TestNamedTensor(TestCase):
             lambda x: x.chunk(2, 0),
         ]
 
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             orig_tensor = torch.empty(2, 2, names=('N', 'D'), device=device)
             for fn in fns:
                 splits = fn(orig_tensor)
@@ -646,13 +655,13 @@ class TestNamedTensor(TestCase):
                     self.assertEqual(split.names, orig_tensor.names)
 
     def test_any_all(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             x = torch.zeros(3, dtype=torch.bool, device=device, names=('C',))
             self.assertEqual(x.any().names, [])
             self.assertEqual(x.all().names, [])
 
     def test_addcmul_addcdiv(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             names = ['N']
             a = torch.rand(3, device=device, names=names)
             b = torch.rand(3, device=device, names=names)
@@ -677,10 +686,12 @@ class TestNamedTensor(TestCase):
 
             self.assertEqual(op(a, a).names, ('N', 'C'))
             self.assertEqual(op(a, c).names, ('N', 'C'))
-
-            with self.assertRaisesRegex(RuntimeError, "do not match"):
+            # TODO: dynamo will throw a slightly different
+            # error message because it's adding fake tensors
+            # `must match the size of` portion is the dynamo error
+            with self.assertRaisesRegex(RuntimeError, "do not match|must match the size of"):
                 op(a, d)
-            with self.assertRaisesRegex(RuntimeError, "do not match"):
+            with self.assertRaisesRegex(RuntimeError, "do not match|must match the size of"):
                 op(a, b)
 
         def test_wildcard(op):
@@ -789,7 +800,7 @@ class TestNamedTensor(TestCase):
     def test_pow_special(self):
         # There are a few pow cases that don't go through TensorIterator.
         # Test them here.
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             named = torch.randn(2, 3, names=('N', 'C'), device=device)
             unnamed = torch.randn([0], device=device)
 
@@ -842,7 +853,7 @@ class TestNamedTensor(TestCase):
                 out = testcase.lambd(tensor)
             except RuntimeError as err:
                 # Get a better error message by catching the error and asserting.
-                raise RuntimeError('{}: {}'.format(testcase.name, err)) from err
+                raise RuntimeError(f'{testcase.name}: {err}') from err
             self.assertEqual(out.names, tensor.names,
                              msg=testcase.name)
 
@@ -979,12 +990,12 @@ class TestNamedTensor(TestCase):
         ]
         tests = flatten(tests)
 
-        for testcase, device in itertools.product(tests, torch.testing.get_all_device_types()):
+        for testcase, device in itertools.product(tests, get_all_device_types()):
             _test(testcase, device=device)
 
     def test_cummax_cummin(self):
         def test_ops(op):
-            for device in torch.testing.get_all_device_types():
+            for device in get_all_device_types():
                 names = ('N', 'D')
                 tensor = torch.rand(2, 3, names=names)
                 result = op(tensor, 0)
@@ -994,14 +1005,14 @@ class TestNamedTensor(TestCase):
         test_ops(torch.cummin)
 
     def test_logcumsumexp(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             names = ('N', 'D')
             tensor = torch.rand(2, 3, names=names)
             result = torch.logcumsumexp(tensor, 'D')
             self.assertEqual(result.names, names)
 
     def test_bitwise_not(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             names = ('N', 'D')
             tensor = torch.zeros(2, 3, names=names, dtype=torch.bool)
             result = torch.empty(0, dtype=torch.bool)
@@ -1011,7 +1022,7 @@ class TestNamedTensor(TestCase):
             self.assertEqual(tensor.bitwise_not_().names, names)
 
     def test_logical_not(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             names = ('N', 'D')
             tensor = torch.zeros(2, 3, names=names, dtype=torch.bool)
             result = torch.empty(0, dtype=torch.bool)
@@ -1021,7 +1032,7 @@ class TestNamedTensor(TestCase):
             self.assertEqual(tensor.logical_not_().names, names)
 
     def test_bernoulli(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             names = ('N', 'D')
             tensor = torch.rand(2, 3, names=names)
             result = torch.empty(0)
@@ -1063,25 +1074,48 @@ class TestNamedTensor(TestCase):
         with self.assertRaisesRegex(RuntimeError, "must be consecutive in"):
             tensor.flatten(['H', 'D', 'W'], 'features')
 
+    def test_flatten_nodims(self):
+        tensor = torch.empty((2, 3))
+        with self.assertRaisesRegex(RuntimeError, "cannot be empty"):
+            tensor.flatten((), 'abcd')
+
+    def test_flatten_index_error(self):
+        tensor = torch.randn(1, 2)
+        with self.assertRaisesRegex(IndexError,
+                                    r"Dimension out of range \(expected to be in range of \[-2, 1\], but got 2\)"):
+            tensor.flatten(0, 2)
+        with self.assertRaisesRegex(IndexError,
+                                    r"Dimension out of range \(expected to be in range of \[-2, 1\], but got 2\)"):
+            tensor.flatten(0, 2, 'N')
+        with self.assertRaisesRegex(RuntimeError,
+                                    r"flatten\(\) has invalid args: start_dim cannot come after end_dim"):
+            tensor.flatten(1, 0)
+        with self.assertRaisesRegex(RuntimeError,
+                                    r"flatten\(\) has invalid args: start_dim cannot come after end_dim"):
+            tensor.flatten(1, 0, 'N')
+
     def test_unflatten(self):
         # test args: tensor, int, namedshape
         self.assertTrue(torch.equal(
-            torch.ones(4).unflatten(0, (('A', 2), ('B', 2))),
+            torch.ones(4, names=('A',)).unflatten('A', (('A', 2), ('B', 2))),
             torch.ones(2, 2, names=('A', 'B'))))
         self.assertTrue(torch.equal(
-            torch.ones(4).unflatten(0, [('A', 2), ('B', 2)]),
+            torch.ones(4, names=('A',)).unflatten('A', [('A', 2), ('B', 2)]),
             torch.ones(2, 2, names=('A', 'B'))))
         self.assertTrue(torch.equal(
-            torch.ones(4).unflatten(0, (['A', 2], ['B', 2])),
+            torch.ones(4, names=('A',)).unflatten('A', (['A', 2], ['B', 2])),
             torch.ones(2, 2, names=('A', 'B'))))
         self.assertTrue(torch.equal(
-            torch.ones(4).unflatten(-1, (['A', 2], ['B', 2])),
-            torch.ones(2, 2, names=('A', 'B'))))
-
-        # test args: namedtensor, int, namedshape
+            torch.ones(2, 10, names=('A', 'B')).unflatten('B', (['B1', -1],)),
+            torch.ones(2, 10, names=('A', 'B1'))))
         self.assertTrue(torch.equal(
-            torch.ones(2, 4, names=('A', 'B')).unflatten(1, (('B1', 2), ('B2', 2))),
-            torch.ones(2, 2, 2, names=('A', 'B1', 'B2'))))
+            torch.ones(2, 3 * 4 * 5 * 6, names=('A', 'B'))
+                 .unflatten('B', (['B1', 3], ['B2', 4], ['B3', -1], ['B4', 6])),
+            torch.ones(2, 3, 4, 5, 6, names=('A', 'B1', 'B2', 'B3', 'B4'))))
+        self.assertTrue(torch.equal(
+            torch.ones(2, 0, names=('A', 'B'))
+                 .unflatten('B', (['B1', 3], ['B2', -1], ['B3', 4])),
+            torch.ones(2, 3, 0, 4, names=('A', 'B1', 'B2', 'B3'))))
 
         # test args: namedtensor, str, namedshape
         self.assertTrue(torch.equal(
@@ -1089,13 +1123,22 @@ class TestNamedTensor(TestCase):
             torch.ones(2, 2, 2, names=('A', 'B1', 'B2'))))
 
         # test invalid args: namedtensor, str, sizes
-        with self.assertRaisesRegex(TypeError, r"received an invalid combination of arguments"):
+        with self.assertRaisesRegex(TypeError, r"unflatten\(\): argument 'dim' \(position 1\) must be int, not str"):
             torch.tensor([1], names=('A',)).unflatten('A', (1, 1))
 
         # test invalid args: namedtensor, int, sizes
         with self.assertRaisesRegex(RuntimeError, r"input is a named tensor but no names were given for unflattened sizes"):
             torch.tensor([1], names=("A",)).unflatten(0, (1, 1))
 
+        with self.assertRaisesRegex(RuntimeError,
+                                    r"Provided sizes \[3, -1\] don't multiply up to the "
+                                    r"size of dim 1 \('B': 4\) in Tensor\['A', 'B'\]"):
+            torch.ones(2, 4, names=('A', 'B')).unflatten('B', (('B1', 3), ('B2', -1)))
+
+        with self.assertRaisesRegex(RuntimeError,
+                                    r"the unspecified dimension size -1 can be any value and is ambiguous"):
+            torch.ones(2, 0, names=('A', 'B')).unflatten('B', (('B1', 0), ('B2', -1)))
+
         tensor = torch.randn(7, 2 * 3 * 5, 11, names=('N', 'D', 'K'))
 
         # accepts OrderedDict
@@ -1152,8 +1195,21 @@ class TestNamedTensor(TestCase):
             check_output(op(t, 1), ['N', 'L'])
             check_output(op(t, -1), ['N', 'C'])
             check_output(op(t, 'C'), ['N', 'L'])
-            with self.assertRaisesRegex(RuntimeError, 'Please look up dimensions by name'):
-                op(t, None)
+            ops_support_dim_none = [
+                'sum',
+                'mean',
+                'std',
+                'var',
+                'std_mean',
+                'var_mean',
+                'nanmean',
+                'nansum',
+            ]
+            if op.__name__ in ops_support_dim_none:
+                check_output(op(t, None), [])
+            else:
+                with self.assertRaisesRegex(RuntimeError, 'Please look up dimensions by name'):
+                    op(t, None)
             with self.assertRaisesRegex(RuntimeError, 'Name \'H\' not found'):
                 op(t, 'H')
 
@@ -1223,7 +1279,7 @@ class TestNamedTensor(TestCase):
             Case(torch.nanmedian, True, False, True, True, values_and_indices),
         ]
 
-        for testcase, device in itertools.product(tests, torch.testing.get_all_device_types()):
+        for testcase, device in itertools.product(tests, get_all_device_types()):
             op = testcase.op
             test_simple_reduce(op, device)
             test_autograd_supports_dimname_overload(op, device)
@@ -1351,6 +1407,8 @@ class TestNamedTensor(TestCase):
         new_refcnt = sys.getrefcount(seen_name)
         self.assertEqual(new_refcnt, old_refcnt)
 
+    # This test is failing on Python 3.12: https://github.com/pytorch/pytorch/issues/119464
+    @unittest.skipIf(sys.version_info >= (3, 12), "Failing on python 3.12+")
     def test_using_unseen_interned_string_bumps_refcount_permanently(self):
         # Please don't use this as a name in a different test.
         unseen_name = 'abcdefghi'
@@ -1361,6 +1419,8 @@ class TestNamedTensor(TestCase):
         new_refcnt = sys.getrefcount(unseen_name)
         self.assertEqual(new_refcnt, old_refcnt + 1)
 
+    # This test is failing on Python 3.12: https://github.com/pytorch/pytorch/issues/119464
+    @unittest.skipIf(sys.version_info >= (3, 12), "Failing on python 3.12+")
     def test_using_unseen_uninterned_string_refcounts(self):
         # Please don't use this as a name in a different test.
         # non-compile-time constants are not interned
@@ -1668,7 +1728,7 @@ class TestNamedTensor(TestCase):
             self.assertTensorDataAndNamesEqual(tensor, expected)
 
     def test_mm(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             self._test_name_inference(
                 torch.mm, device=device,
                 args=(create('N:3,C:2'), create('W:2,H:5')),
@@ -1698,10 +1758,10 @@ class TestNamedTensor(TestCase):
                 maybe_raises_regex='with duplicate names')
 
     def test_expand(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             self._test_name_inference(
                 Tensor.expand, device=device,
-                args=(create('D:1'), [3]), expected_names=('D'))
+                args=(create('D:1'), [3]), expected_names=('D',))
 
             self._test_name_inference(
                 Tensor.expand, device=device,
@@ -1714,7 +1774,7 @@ class TestNamedTensor(TestCase):
                 expected_names=(None, None, None, None))
 
     def test_addmm(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             # full names
             self._test_name_inference(
                 torch.addmm, device=device,
@@ -1751,7 +1811,7 @@ class TestNamedTensor(TestCase):
                 maybe_raises_regex='with duplicate names')
 
     def test_bmm(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             # full names
             self._test_name_inference(
                 torch.bmm, device=device,
@@ -1795,7 +1855,7 @@ class TestNamedTensor(TestCase):
                 maybe_raises_regex='misaligned')
 
     def test_matmul(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             # input tensors are less than 1D
             self._test_name_inference(
                 torch.matmul, device=device,
@@ -1875,7 +1935,7 @@ class TestNamedTensor(TestCase):
                 maybe_raises_regex='do not match')
 
     def test_mv(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             self._test_name_inference(
                 torch.mv, device=device,
                 args=(create('N:3,C:2'), create('W:2')),
@@ -1900,7 +1960,7 @@ class TestNamedTensor(TestCase):
                 expected_names=('N',))
 
     def test_addmv(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             # full names
             self._test_name_inference(
                 torch.addmv, device=device,
@@ -1960,7 +2020,7 @@ class TestNamedTensor(TestCase):
             x.squeeze_("N")
 
     def test_dot(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             # torch.dot ignores the names of both tensors
             self._test_name_inference(
                 torch.dot, device=device,
@@ -1968,7 +2028,7 @@ class TestNamedTensor(TestCase):
                 expected_names=[])
 
     def test_comparison_ops(self):
-        for device in torch.testing.get_all_device_types():
+        for device in get_all_device_types():
             a = torch.randn(3, 3, names=('N', 'C'), device=device)
             b = torch.randn(3, 3, names=('N', 'C'), device=device)
             scalar = torch.randn([], device=device)
@@ -2014,6 +2074,13 @@ class TestNamedTensor(TestCase):
             res = torch.isinf(a)
             self.assertEqual(res.names, ['N', 'C'])
 
+    def test_support_device_named_grad(self):
+        named_tensor = torch.randn(3, 3, device='meta')
+        with self.assertRaisesRegex(RuntimeError, 'NYI: named tensors only support CPU, CUDA'):
+            named_tensor.rename_('N', 'C')
+            named_tensor.names = ['N', 'C']
+            named_tensor = torch.randn(3, 3, device='meta', names=['N', 'C'])
+
 
 if __name__ == '__main__':
     run_tests()
