@@ -1,14 +1,20 @@
-import unittest
-from torch.testing._internal.common_utils import TestCase, run_tests, set_cwd
-from torch.testing._internal.mypy_wrapper import config_files
-import tempfile
-import torch
+# mypy: allow-untyped-defs
+# Owner(s): ["module: typing"]
+
 import doctest
-import os
 import inspect
+import os
+import tempfile
+import unittest
+from pathlib import Path
+
+import torch
+from torch.testing._internal.common_utils import run_tests, set_cwd, TestCase
+
 
 try:
     import mypy.api
+
     HAVE_MYPY = True
 except ImportError:
     HAVE_MYPY = False
@@ -20,7 +26,7 @@ def get_examples_from_docstring(docstr):
     in docstrings; returns a list of lines.
     """
     examples = doctest.DocTestParser().get_examples(docstr)
-    return [f'    {l}' for e in examples for l in e.source.splitlines()]
+    return [f"    {l}" for e in examples for l in e.source.splitlines()]
 
 
 def get_all_examples():
@@ -31,16 +37,20 @@ def get_all_examples():
     """
     blocklist = {
         "_np",
+        "_InputT",
     }
-    allexamples = ""
 
     example_file_lines = [
+        "# mypy: allow-untyped-defs",
+        "",
+        "import math",
+        "import io",
+        "import itertools",
+        "",
+        "import numpy",
+        "",
         "import torch",
         "import torch.nn.functional as F",
-        "import math  # type: ignore",  # mypy complains about floats where SupportFloat is expected
-        "import numpy  # type: ignore",
-        "import io  # type: ignore",
-        "import itertools  # type: ignore",
         "",
         # for requires_grad_ example
         # NB: We are parsing this file as Python 2, so we must use
@@ -56,7 +66,7 @@ def get_all_examples():
         if docstr and fname not in blocklist:
             e = get_examples_from_docstring(docstr)
             if e:
-                example_file_lines.append(f"\n\ndef example_torch_{fname}():")
+                example_file_lines.append(f"\n\ndef example_torch_{fname}() -> None:")
                 example_file_lines += e
 
     for fname in dir(torch.Tensor):
@@ -65,7 +75,9 @@ def get_all_examples():
         if docstr and fname not in blocklist:
             e = get_examples_from_docstring(docstr)
             if e:
-                example_file_lines.append(f"\n\ndef example_torch_tensor_{fname}():")
+                example_file_lines.append(
+                    f"\n\ndef example_torch_tensor_{fname}() -> None:"
+                )
                 example_file_lines += e
 
     return "\n".join(example_file_lines)
@@ -77,9 +89,8 @@ class TestTypeHints(TestCase):
         """
         Run documentation examples through mypy.
         """
-        fn = os.path.join(os.path.dirname(__file__), 'generated_type_hints_smoketest.py')
-        with open(fn, "w") as f:
-            print(get_all_examples(), file=f)
+        fn = Path(__file__).resolve().parent / "generated_type_hints_smoketest.py"
+        fn.write_text(get_all_examples())
 
         # OK, so here's the deal.  mypy treats installed packages
         # and local modules differently: if a package is installed,
@@ -114,61 +125,25 @@ class TestTypeHints(TestCase):
             try:
                 os.symlink(
                     os.path.dirname(torch.__file__),
-                    os.path.join(tmp_dir, 'torch'),
-                    target_is_directory=True
+                    os.path.join(tmp_dir, "torch"),
+                    target_is_directory=True,
                 )
             except OSError:
-                raise unittest.SkipTest('cannot symlink') from None
-            (stdout, stderr, result) = mypy.api.run([
-                '--cache-dir=.mypy_cache/doc',
-                '--no-strict-optional',  # needed because of torch.lu_unpack, see gh-36584
-                os.path.abspath(fn),
-            ])
+                raise unittest.SkipTest("cannot symlink") from None
+            repo_rootdir = Path(__file__).resolve().parent.parent
+            # TODO: Would be better not to chdir here, this affects the
+            # entire process!
+            with set_cwd(str(repo_rootdir)):
+                (stdout, stderr, result) = mypy.api.run(
+                    [
+                        "--cache-dir=.mypy_cache/doc",
+                        "--no-strict-optional",  # needed because of torch.lu_unpack, see gh-36584
+                        str(fn),
+                    ]
+                )
             if result != 0:
-                self.fail(f"mypy failed:\n{stdout}")
+                self.fail(f"mypy failed:\n{stderr}\n{stdout}")
 
-    @unittest.skipIf(not HAVE_MYPY, "need mypy")
-    def test_run_mypy(self):
-        """
-        Runs mypy over all files specified in our mypy configs
-        Note that our mypy configs are not shipped in an installed
-        version of PyTorch, so this test will only run mypy in a
-        development setup or in CI.
-        """
-        def is_torch_mypyini(path_to_file):
-            with open(path_to_file, 'r') as f:
-                first_line = f.readline()
 
-            name = os.path.basename(path_to_file)
-            if first_line.startswith(f'# This is the PyTorch {name} file'):
-                return True
-
-            return False
-
-        # to add more configs, edit the implementation of the
-        # config_files function rather than editing this test or adding
-        # more tests to this suite
-        for ini in config_files():
-            with self.subTest(msg=ini):
-                test_dir = os.path.dirname(os.path.realpath(__file__))
-                repo_rootdir = os.path.join(test_dir, '..')
-                mypy_inifile = os.path.join(repo_rootdir, ini)
-                if not (os.path.exists(mypy_inifile) and is_torch_mypyini(mypy_inifile)):
-                    self.skipTest("Can't find PyTorch MyPy config file")
-
-                import numpy
-                if numpy.__version__.startswith('1.20.0.dev0'):
-                    self.skipTest("Typeannotations in numpy-1.20.0-dev are broken")
-
-                # TODO: Would be better not to chdir here, this affects
-                # the entire process!
-                with set_cwd(repo_rootdir):
-                    (stdout, stderr, result) = mypy.api.run([
-                        '--config', mypy_inifile,
-                    ])
-
-                if result != 0:
-                    self.fail(f"mypy failed: {stdout} {stderr}")
-
-if __name__ == '__main__':
+if __name__ == "__main__":
     run_tests()
