@@ -16,9 +16,10 @@ def bracket_pairs(tokens: Sequence[TokenInfo]) -> dict[int, int]:
     """Returns a dictionary mapping opening to closing brackets"""
     braces: dict[int, int] = {}
     stack: list[int] = []
+    in_fstring = False
 
     for i, t in enumerate(tokens):
-        if t.type == token.OP:
+        if t.type == token.OP and not in_fstring:
             if t.string in BRACKETS:
                 stack.append(i)
             elif inv := BRACKETS_INV.get(t.string):
@@ -34,9 +35,11 @@ def bracket_pairs(tokens: Sequence[TokenInfo]) -> dict[int, int]:
                     raise ParseError(t, f"Mismatched braces '{b}' at {begin}")
         elif t.type == FSTRING_START:
             stack.append(FSTRING_START)
+            in_fstring = True
         elif t.type == FSTRING_END:
             if stack.pop() != FSTRING_START:
                 raise ParseError(t, "Mismatched FSTRING_START/FSTRING_END")
+            in_fstring = False
     if stack:
         raise ParseError(t, "Left open")
     return braces
