@@ -1,15 +1,10 @@
+#include <ATen/MapAllocator.h>
+#include <c10/cuda/CUDAGuard.h>
 #include <torch/csrc/CudaIPCTypes.h>
-#include <TH/THAllocator.h>
+#include <atomic>
 #include <map>
 #include <mutex>
-#include <random>
-
-#ifdef _MSC_VER
-#include <c10/util/win32-headers.h>
-#else
-#include <sys/types.h>
-#include <unistd.h>
-#endif
+#include <string>
 
 namespace torch {
 
@@ -25,19 +20,33 @@ void warnProducerTerminatedBeforeSharedTensorsReleased() {
 }
 
 struct CudaIPCGlobalEntities {
+  // This class is used as a singleton (see cuda_ipc_global_entities)
+  // This variable is used to track its lifetime to avoid accessing it
+  // after it was destroyed which would lead to segmentation faults
+  // Note that a trvial type is used which doesn't suffer from construction
+  // and destruction order issues
+  static bool alive;
+
   std::mutex ref_counters_mutex_;
-  std::atomic<int64_t> sync_events_used_;
+  std::atomic<int64_t> sync_events_used_{0};
   std::map<std::string, std::shared_ptr<CudaIPCRefCountersFile>>
       ref_counters_files_;
   std::shared_ptr<CudaIPCRefCountersFile> next_available_ref_counters_file_;
   CudaIPCSentDataLimbo CudaIPCSentDataLimbo_;
-  CudaIPCGlobalEntities() : ref_counters_files_() {}
+  CudaIPCGlobalEntities() {
+    alive = true;
+  }
+  CudaIPCGlobalEntities(const CudaIPCGlobalEntities&) = delete;
+  CudaIPCGlobalEntities(CudaIPCGlobalEntities&&) = delete;
+  CudaIPCGlobalEntities& operator=(const CudaIPCGlobalEntities&) = delete;
+  CudaIPCGlobalEntities& operator=(CudaIPCGlobalEntities&&) = delete;
   ~CudaIPCGlobalEntities() {
     CudaIPCSentDataLimbo_.collect();
     safe_clean_current_file();
     if (next_available_ref_counters_file_) {
       warnProducerTerminatedBeforeSharedTensorsReleased();
     }
+    alive = false;
   }
   void safe_clean_current_file() {
     std::lock_guard<std::mutex> lock(ref_counters_mutex_);
@@ -49,11 +58,12 @@ struct CudaIPCGlobalEntities {
   }
 };
 
+bool CudaIPCGlobalEntities::alive = false;
 CudaIPCGlobalEntities cuda_ipc_global_entities;
 
 CudaIPCSentDataLimbo::~CudaIPCSentDataLimbo() {
   collect();
-  if (shared_blocks_.size() > 0) {
+  if (size() > 0) {
     warnProducerTerminatedBeforeSharedTensorsReleased();
   }
 }
@@ -74,7 +84,8 @@ bool CudaIPCSentDataLimbo::collect() {
     }
     shared_blocks_ = std::move(kept_blocks);
   }
-  // Need to reset blocks out of the critical section here, otherwise it deadlocks.
+  // Need to reset blocks out of the critical section here, otherwise it
+  // deadlocks.
   for (auto& sd : reset_blocks) {
     sd.reset();
   }
@@ -96,9 +107,17 @@ void CudaIPCSentDataLimbo::add(std::unique_ptr<CudaIPCSentData> shared_block) {
   shared_blocks_.push_back(std::move(shared_block));
 }
 
+uint64_t CudaIPCSentDataLimbo::size() {
+  std::lock_guard<std::mutex> lock(limbo_mutex_);
+  return shared_blocks_.size();
+}
+
 void CudaIPCSentDataDelete(void* ptr) {
   std::unique_ptr<CudaIPCSentData> sent_data(
       static_cast<CudaIPCSentData*>(ptr));
+  if (!CudaIPCGlobalEntities::alive) {
+    return;
+  }
   if (sent_data->counter_value() > 0) {
     cuda_ipc_global_entities.CudaIPCSentDataLimbo_.add(std::move(sent_data));
   }
@@ -106,13 +125,18 @@ void CudaIPCSentDataDelete(void* ptr) {
 }
 
 void ReturnRefCounter(const std::string& handle, uint64_t offset /* unused */) {
+  if (!CudaIPCGlobalEntities::alive) {
+    return;
+  }
   std::lock_guard<std::mutex> lock(
       cuda_ipc_global_entities.ref_counters_mutex_);
-  cuda_ipc_global_entities.ref_counters_files_[handle]->return_offset(offset);
-  if (cuda_ipc_global_entities.ref_counters_files_[handle]->offsets_in_use() ==
-          0 &&
-      !cuda_ipc_global_entities.ref_counters_files_[handle]->have_offsets()) {
-    cuda_ipc_global_entities.ref_counters_files_.erase(handle);
+  auto& map = cuda_ipc_global_entities.ref_counters_files_;
+  auto it = map.find(handle);
+  if (it != map.end()) {
+    it->second->return_offset(offset);
+    if (it->second->offsets_in_use() == 0 && !it->second->have_offsets()) {
+      map.erase(handle);
+    }
   }
 }
 
@@ -120,32 +144,33 @@ void ReturnRefCounter(const std::string& handle, uint64_t offset /* unused */) {
 
 CudaIPCSentData::CudaIPCSentData(
     std::string handle,
-    int64_t offset,
-    int64_t* counter_ptr,
+    uint64_t offset,
+    uint64_t* counter_ptr,
     at::Device device)
-    : handle_(handle),
+    : handle_(std::move(handle)),
       offset_(offset),
       counter_ptr_(counter_ptr),
-      original_ptr_(),
       device_(device) {
-#ifndef __HIP_PLATFORM_HCC__
-  // CUDA have the unofficial limit on the number of recorded blocking interprocess
-  // events, to prevent using of all events, we are switching to StreamSync
-  // before limit reached.
+#if !defined(USE_ROCM)
+  // CUDA have the unofficial limit on the number of recorded blocking
+  // interprocess events, to prevent using of all events, we are switching to
+  // StreamSync before limit reached.
   //
   //  ```python
   //  import torch
   //  a = [ torch.cuda.Event(
-  //      enable_timing=False, blocking=True, interprocess=True) for i in range(30000) ]
+  //      enable_timing=False, blocking=True, interprocess=True) for i in
+  //      range(30000) ]
   //  [i.record() for i in a]
   //  ```
   //
-  if (cuda_ipc_global_entities.sync_events_used_.load() < CUDA_IPC_MAXIMUM_EVENTS_TO_USE) {
+  if (cuda_ipc_global_entities.sync_events_used_.load() <
+      CUDA_IPC_MAXIMUM_EVENTS_TO_USE) {
     // TODO: More efficient would be to create event inside of main thread (at
     // the moment of the queue.put). The reason this is more efficient is
     // because the main thread may have queued extra work on the stream, which
     // this event will consequently wait for (uselessly).
-    cuda_ipc_global_entities.sync_events_used_ ++;
+    cuda_ipc_global_entities.sync_events_used_++;
     C10_CUDA_CHECK(cudaEventCreateWithFlags(
         &event_,
         cudaEventDisableTiming | cudaEventInterprocess |
@@ -155,33 +180,38 @@ CudaIPCSentData::CudaIPCSentData(
     event_sync_required_ = true;
   } else {
     auto stream = c10::cuda::getCurrentCUDAStream(device.index());
-    C10_CUDA_CHECK(cudaStreamSynchronize(stream));
+    at::cuda::stream_synchronize(stream);
+    event_ = nullptr;
     event_sync_required_ = false;
   }
 #else
   // cuIpcGetEventHandle with HIP is not supported, so we have to sync
   // stream instead of passing event
   auto stream = c10::cuda::getCurrentCUDAStream(device.index());
-  C10_CUDA_CHECK(cudaStreamSynchronize(stream));
+  at::cuda::stream_synchronize(stream);
   event_sync_required_ = false;
 #endif
 }
 
 CudaIPCSentData::~CudaIPCSentData() {
   ReturnRefCounter(handle_, offset_);
-#ifndef __HIP_PLATFORM_HCC__
+#if !defined(USE_ROCM)
   try {
     if (event_sync_required_) {
       at::cuda::CUDAGuard device_guard(device_.index());
-      cudaEventDestroy(event_);
-      cuda_ipc_global_entities.sync_events_used_ --;
+      C10_CUDA_CHECK(cudaEventDestroy(event_));
+      if (!CudaIPCGlobalEntities::alive) {
+        return;
+      }
+      cuda_ipc_global_entities.sync_events_used_--;
     }
+    // NOLINTNEXTLINE(bugprone-empty-catch)
   } catch (...) { /* No throw */
   }
 #endif
 }
 
-int64_t CudaIPCSentData::counter_value() {
+uint64_t CudaIPCSentData::counter_value() {
   return *counter_ptr_;
 }
 
@@ -190,18 +220,11 @@ at::DataPtr GetNewRefCountedSentData(void* data, at::Device device) {
     std::lock_guard<std::mutex> lock(
         cuda_ipc_global_entities.ref_counters_mutex_);
     if (!cuda_ipc_global_entities.next_available_ref_counters_file_) {
-      static std::random_device rd;
-      std::string ref_counter_handle = "/torch_";
-#ifdef _MSC_VER
-      ref_counter_handle += std::to_string(GetCurrentProcessId());
-#else
-      ref_counter_handle += std::to_string(getpid());
-#endif
-      ref_counter_handle += "_";
-      ref_counter_handle += std::to_string(rd());
+      std::string ref_counter_handle = at::NewProcessWideShmHandle();
 
-      int flags = TH_ALLOCATOR_MAPPED_SHAREDMEM | TH_ALLOCATOR_MAPPED_EXCLUSIVE;
-      at::DataPtr sptr = THRefcountedMapAllocator::makeDataPtr(
+      int flags =
+          at::ALLOCATOR_MAPPED_SHAREDMEM | at::ALLOCATOR_MAPPED_EXCLUSIVE;
+      at::DataPtr sptr = at::RefcountedMapAllocator::makeDataPtr(
           ref_counter_handle.c_str(),
           flags,
           sizeof(int64_t) * CUDA_IPC_REF_COUNTER_FILE_SIZE,
@@ -228,6 +251,9 @@ at::DataPtr GetNewRefCountedSentData(void* data, at::Device device) {
 }
 
 bool CudaIPCCollect() {
+  if (!CudaIPCGlobalEntities::alive) {
+    return true;
+  }
   bool freed_memory = cuda_ipc_global_entities.CudaIPCSentDataLimbo_.collect();
   if (cuda_ipc_global_entities.CudaIPCSentDataLimbo_.size() == 0) {
     cuda_ipc_global_entities.safe_clean_current_file();
@@ -239,6 +265,6 @@ bool CudaIPCCollect() {
 
 namespace c10 {
 namespace {
-REGISTER_FREE_MEMORY_CALLBACK("cuda_ipc_collect", CudaIPCCollectCallback);
+REGISTER_FREE_MEMORY_CALLBACK("cuda_ipc_collect", CudaIPCCollectCallback)
 }
 } // namespace c10
