@@ -3,6 +3,8 @@
 #include <torch/csrc/inductor/aoti_torch/utils.h>
 #include <ATen/mps/MPSAllocatorInterface.h>
 #include <ATen/mps/MPSDevice.h>
+#include <ATen/mps/MPSStream.h>
+#include <ATen/mps/MPSProfiler.h>
 
 
 using namespace torch::aot_inductor;
@@ -40,3 +42,16 @@ aoti_torch_mps_memcpy(void* buffer, size_t constant_offset, size_t bytes_read, s
     memcpy(buffer_pointer + constant_offset, constants_start + bytes_read, data_size);
   });
 }
+
+AOTITorchError
+aoti_torch_mps_copy_buffer(void* src_buffer, void* dst_buffer, size_t data_size, size_t src_offset, size_t dst_offset) {
+  AOTI_TORCH_CONVERT_EXCEPTION_TO_ERROR_CODE({
+
+    auto src_mtl_buffer = (id<MTLBuffer>)src_buffer;
+    auto dst_mtl_buffer = (id<MTLBuffer>)dst_buffer;
+
+    auto* stream = at::mps::getCurrentMPSStream();
+    uint64_t profile_id = at::mps::getMPSProfiler().beginProfileCopy(src_mtl_buffer, dst_mtl_buffer, at::OptionalTensorRef(), at::OptionalTensorRef(), data_size, true);
+    stream->copy_and_sync(src_mtl_buffer, dst_mtl_buffer, data_size, src_offset, dst_offset, true, profile_id);
+  });
+}
