@@ -2,11 +2,11 @@
 
 #include <torch/csrc/jit/ir/ir.h>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 // Inline function and method calls.
 TORCH_API void Inline(Graph& graph);
 
-} // namespace jit
-} // namespace torch
+TORCH_API GraphFunction* tryToGraphFunction(Node* n);
+
+} // namespace torch::jit
