@@ -3,10 +3,10 @@
 #include <torch/csrc/jit/api/module.h>
 #include <torch/csrc/jit/ir/ir.h>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
+
+TORCH_API void removeDropout(std::shared_ptr<Graph>& graph);
 
 TORCH_API void removeDropout(script::Module& module);
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
