@@ -1,14 +1,13 @@
 #include <torch/csrc/jit/tensorexpr/registerizer.h>
+#include <iostream>
 
-namespace torch {
-namespace jit {
-namespace tensorexpr {
+namespace torch::jit::tensorexpr {
 namespace registerizer {
 
 // AccessInfo
 
 void AccessInfo::addStore(
-    const Store* store,
+    const StorePtr& store,
     const std::shared_ptr<Scope>& scope) {
   block_ =
       block_ ? Block::getSharedParent(block_, scope->block()) : scope->block();
@@ -19,7 +18,8 @@ void AccessInfo::addStore(
   first_usage_ = first_usage_ ? block_->getEnclosedRoot(first_usage_) : store;
   last_usage_ = store;
 
-  store_cost_ = IRSimplifier::simplify(new Add(store_cost_, new IntImm(1)));
+  store_cost_ =
+      IRSimplifier::simplify(alloc<Add>(store_cost_, immLike(store_cost_, 1)));
   stores_.push_back(store);
 
   conditionId_ = scope->conditionId();
@@ -27,15 +27,16 @@ void AccessInfo::addStore(
 }
 
 void AccessInfo::addLoad(
-    const Load* load,
+    const LoadPtr& load,
     const std::shared_ptr<Scope>& scope,
-    const Stmt* usage) {
+    const StmtPtr& usage) {
   block_ =
       block_ ? Block::getSharedParent(block_, scope->block()) : scope->block();
   first_usage_ = first_usage_ ? block_->getEnclosedRoot(first_usage_) : usage;
   last_usage_ = usage;
 
-  load_cost_ = IRSimplifier::simplify(new Add(load_cost_, new IntImm(1)));
+  load_cost_ =
+      IRSimplifier::simplify(alloc<Add>(load_cost_, immLike(load_cost_, 1)));
   loads_.push_back(load);
 
   conditionId_ = scope->conditionId();
@@ -43,20 +44,27 @@ void AccessInfo::addLoad(
 }
 
 void AccessInfo::merge(const std::shared_ptr<AccessInfo>& other) {
-  TORCH_INTERNAL_ASSERT(hash_ == other->hash());
-  TORCH_INTERNAL_ASSERT(indices_.size() == other->indices().size());
+  TORCH_INTERNAL_ASSERT(
+      hash_ == other->hash(),
+      buildErrorMessage(
+          "Expected hashes to match in registerizer in the fuser."));
+  TORCH_INTERNAL_ASSERT(
+      indices_.size() == other->indices().size(),
+      buildErrorMessage(
+          "Expected ranks to match in registerizer in the fuser."));
 
   last_usage_ = other->last_usage();
-  for (auto* s : other->stores()) {
+  for (const auto& s : other->stores()) {
     stores_.push_back(s);
   }
-  for (auto* l : other->loads()) {
+  for (const auto& l : other->loads()) {
     loads_.push_back(l);
   }
 
   store_cost_ =
-      IRSimplifier::simplify(new Add(store_cost_, other->store_cost()));
-  load_cost_ = IRSimplifier::simplify(new Add(load_cost_, other->load_cost()));
+      IRSimplifier::simplify(alloc<Add>(store_cost_, other->store_cost()));
+  load_cost_ =
+      IRSimplifier::simplify(alloc<Add>(load_cost_, other->load_cost()));
 
   block_ = Block::getSharedParent(block_, other->block());
   // update first and last usage to be in the parent Block.
@@ -67,15 +75,18 @@ void AccessInfo::merge(const std::shared_ptr<AccessInfo>& other) {
 
 bool AccessInfo::overlaps(const std::shared_ptr<AccessInfo>& other) {
   // All accesses to a buf must have the same dimensionality.
-  TORCH_INTERNAL_ASSERT(indices_.size() == other->indices().size());
+  TORCH_INTERNAL_ASSERT(
+      indices_.size() == other->indices().size(),
+      buildErrorMessage(
+          "Expected ranks to match in registerizer in the fuser."));
 
-  const auto& other_indices = other->indices();
+  auto& other_indices = other->indices();
 
   // They don't overlap if there is a guaranteed difference in any
   // dimension.
   bool overlap = true;
   for (size_t i = 0; i < indices_.size(); ++i) {
-    const Expr* diff = new Sub(indices_[i], other_indices[i]);
+    ExprPtr diff = alloc<Sub>(indices_[i], other_indices[i]);
     diff = IRSimplifier::simplify(diff);
 
     if (diff->isConstant() && !immediateEquals(diff, 0)) {
@@ -87,9 +98,9 @@ bool AccessInfo::overlaps(const std::shared_ptr<AccessInfo>& other) {
   return overlap;
 }
 
-bool AccessInfo::dependsOnVar(const Var* v) {
+bool AccessInfo::dependsOnVar(const VarPtr& v) {
   VarFinder vf;
-  for (auto* i : indices_) {
+  for (const auto& i : indices_) {
     i->accept(&vf);
   }
 
@@ -107,10 +118,10 @@ std::shared_ptr<AccessInfo> AccessInfo::cloneWithHiddenInfo(
   newInfo->firstUsageOverlapped_ = orig->firstUsageOverlapped_;
   newInfo->store_cost_ = orig->store_cost_;
   newInfo->load_cost_ = orig->load_cost_;
-  for (auto* s : orig->stores_) {
+  for (const auto& s : orig->stores_) {
     newInfo->stores_.push_back(s);
   }
-  for (auto* s : orig->loads_) {
+  for (const auto& s : orig->loads_) {
     newInfo->loads_.push_back(s);
   }
 
@@ -121,7 +132,7 @@ std::shared_ptr<AccessInfo> AccessInfo::cloneWithHiddenInfo(
 
 void AccessInfo::print() const {
   std::cout << "Access: " << *buf_ << "{";
-  for (auto* i : indices_) {
+  for (const auto& i : indices_) {
     std::cout << *i << " ";
   }
   std::cout << "} stores: " << stores_.size() << " (" << *store_cost_ << ") -";
@@ -139,7 +150,7 @@ void Scope::closeAccess(const std::shared_ptr<AccessInfo>& info) {
   closedAccesses_.push_back(info);
 }
 
-AccessHashMap& Scope::getAccessMapByBuf(const Buf* b) {
+AccessHashMap& Scope::getAccessMapByBuf(const BufPtr& b) {
   auto it = openAccesses_.find(b);
   if (it == openAccesses_.end()) {
     // create and return
@@ -179,7 +190,7 @@ void RegisterizerAnalysis::closeAccessIntoScope(
   scope->closeAccess(info);
 }
 
-void RegisterizerAnalysis::visit(const For* v) {
+void RegisterizerAnalysis::visit(const ForPtr& v) {
   if (v->loop_options().is_gpu_block_index() ||
       v->loop_options().is_gpu_thread_index()) {
     throw malformed_input(
@@ -195,13 +206,12 @@ void RegisterizerAnalysis::visit(const For* v) {
   v->body()->accept(this);
   stmtStack_.pop_front();
 
-  const Expr* loopExtent =
-      IRSimplifier::simplify(new Sub(v->stop(), v->start()));
+  ExprPtr loopExtent =
+      IRSimplifier::simplify(alloc<Sub>(v->stop(), v->start()));
 
   // now we need to see which accesses we can hoist out of the for loop, their
   // costs should be multiplied by the loop extent.
   for (auto& pair : currentScope_->openAccesses()) {
-    const Buf* buf = pair.first;
     if (pair.second.empty()) {
       continue;
     }
@@ -215,7 +225,7 @@ void RegisterizerAnalysis::visit(const For* v) {
       // possible that an access at a higher scope could "unhide" the
       // conditional access, in which case we need to hoist. If there is no
       // access to this element at a higher scope then we cannot safely hoist.
-      // We cannot know at this level whether that will or wont occur.
+      // We cannot know at this level whether that will or won't occur.
       //
       // The solution we take here is to split the space-time continuum, and
       // keep both versions of the access handy. If the hoisted access is not
@@ -228,7 +238,7 @@ void RegisterizerAnalysis::visit(const For* v) {
       bool closed = false;
       // If this access depends on a locally scoped variable, it cannot be
       // hosted out of the loop.
-      for (auto* v : currentScope_->localVars()) {
+      for (const auto& v : currentScope_->localVars()) {
         if (candidate->dependsOnVar(v)) {
           closeAccessIntoScope(candidate, currentScope_);
           closed = true;
@@ -262,12 +272,12 @@ void RegisterizerAnalysis::visit(const For* v) {
 
   // having hoisted, now we can merge normally.
   mergeCurrentScopeIntoParent();
-};
+}
 
-void RegisterizerAnalysis::visit(const Cond* v) {
-  const Expr* condition = v->condition();
-  Block* true_stmt = v->true_stmt();
-  Block* false_stmt = v->false_stmt();
+void RegisterizerAnalysis::visit(const CondPtr& v) {
+  ExprPtr condition = v->condition();
+  BlockPtr true_stmt = v->true_stmt();
+  BlockPtr false_stmt = v->false_stmt();
 
   stmtStack_.push_front(v);
 
@@ -304,10 +314,10 @@ void RegisterizerAnalysis::visit(const Cond* v) {
 // IfThenElses are just like Conds except they are not Stmts, which means no
 // registerization can occur internally. However, the first reference to an
 // access can occur within one if its visible outside the condition.
-void RegisterizerAnalysis::visit(const IfThenElse* v) {
-  const Expr* condition = v->condition();
-  const Expr* true_value = v->true_value();
-  const Expr* false_value = v->false_value();
+void RegisterizerAnalysis::visit(const IfThenElsePtr& v) {
+  ExprPtr condition = v->condition();
+  ExprPtr true_value = v->true_value();
+  ExprPtr false_value = v->false_value();
 
   // condition is in enclosing scope.
   condition->accept(this);
@@ -339,7 +349,7 @@ void RegisterizerAnalysis::visit(const IfThenElse* v) {
   }
 }
 
-void RegisterizerAnalysis::visit(const Let* v) {
+void RegisterizerAnalysis::visit(const LetPtr& v) {
   currentScope_->addLocalVar(v->var());
 
   stmtStack_.push_front(v);
@@ -347,7 +357,7 @@ void RegisterizerAnalysis::visit(const Let* v) {
   stmtStack_.pop_front();
 }
 
-void RegisterizerAnalysis::visit(const Block* v) {
+void RegisterizerAnalysis::visit(const BlockPtr& v) {
   auto prev_scope = currentScope_;
   if (currentScope_->block() != v) {
     currentScope_ = std::make_shared<Scope>(v, prev_scope);
@@ -355,7 +365,7 @@ void RegisterizerAnalysis::visit(const Block* v) {
 
   stmtStack_.push_front(v);
 
-  for (auto* s : *v) {
+  for (const auto& s : *v) {
     s->accept(this);
     if (currentScope_->block() != v) {
       // merge the inner block's accesses into this Block's accesses.
@@ -375,7 +385,7 @@ void RegisterizerAnalysis::visit(const Block* v) {
   }
 }
 
-void RegisterizerAnalysis::visit(const Store* v) {
+void RegisterizerAnalysis::visit(const StorePtr& v) {
   stmtStack_.push_front(v);
   v->value()->accept(this);
   stmtStack_.pop_front();
@@ -387,10 +397,9 @@ void RegisterizerAnalysis::visit(const Store* v) {
 
   // hash the Store:
   SimplifierHashType accessHash = hasher_.hash(v->buf());
-  for (auto* i : v->indices()) {
+  for (const auto& i : v->indices()) {
     accessHash = hasher_.hash_combine(accessHash, i);
   }
-  accessHash = hasher_.hash_combine(accessHash, v->mask());
 
   auto& bufAccesses = currentScope_->getAccessMapByBuf(v->buf());
   auto candidateIt = bufAccesses.find(accessHash);
@@ -430,17 +439,16 @@ void RegisterizerAnalysis::visit(const Store* v) {
   }
 }
 
-void RegisterizerAnalysis::visit(const Load* v) {
+void RegisterizerAnalysis::visit(const LoadPtr& v) {
   if (v->indices().empty()) {
     // already a scalar.
     return;
   }
   // hash the Load:
   SimplifierHashType accessHash = hasher_.hash(v->buf());
-  for (auto* i : v->indices()) {
+  for (const auto& i : v->indices()) {
     accessHash = hasher_.hash_combine(accessHash, i);
   }
-  accessHash = hasher_.hash_combine(accessHash, v->mask());
 
   auto& bufAccesses = currentScope_->getAccessMapByBuf(v->buf());
   auto candidateIt = bufAccesses.find(accessHash);
@@ -480,7 +488,7 @@ void RegisterizerAnalysis::visit(const Load* v) {
 }
 
 // Loop and Conditional scopes are different in that it may or may not be
-// possible to hoist the intializer of a scalar variable outside the block
+// possible to hoist the initializer of a scalar variable outside the block
 // depending on if we can tell that the Buffer access is valid outside. This is
 // tricky because the access that demonstrates this may be later in the tree and
 // we haven't encountered it yet.
@@ -518,7 +526,7 @@ void RegisterizerAnalysis::mergeHiddenScope(bool allowClosed) {
 void RegisterizerAnalysis::mergeCurrentScopeIntoParent() {
   auto parent = currentScope_->parent();
 
-  // copy across current closed accceses, merging / closing as necessary
+  // copy across current closed accesses, merging / closing as necessary
   for (auto& candidate : currentScope_->closedAccesses()) {
     auto& parentAccesses = parent->getAccessMapByBuf(candidate->buf());
 
@@ -534,7 +542,7 @@ void RegisterizerAnalysis::mergeCurrentScopeIntoParent() {
         closeAccessIntoScope(pCandidate, parent);
         parentAccesses.erase(parentIt);
 
-        // the childs access inserted into the parent scope.
+        // the children access inserted into the parent scope.
         closeAccessIntoScope(candidate, parent);
         continue;
       }
@@ -559,14 +567,14 @@ void RegisterizerAnalysis::mergeCurrentScopeIntoParent() {
       ++it;
     }
 
-    // Insert the childs closed access into the parent scope.
+    // Insert the children closed access into the parent scope.
     closeAccessIntoScope(candidate, parent);
   }
 
   // copy across current open accesses, merging as necessary.
   // for each Buf with an open access:
   for (auto& pair : currentScope_->openAccesses()) {
-    const Buf* buf = pair.first;
+    BufPtr buf = pair.first;
     if (pair.second.empty()) {
       continue;
     }
@@ -610,7 +618,7 @@ void RegisterizerAnalysis::mergeCurrentScopeIntoParent() {
 
       // If this access depends on a locally scoped variable, it cannot be
       // lifted out of the loop.
-      for (auto* v : currentScope_->localVars()) {
+      for (const auto& v : currentScope_->localVars()) {
         if (candidate->dependsOnVar(v)) {
           closeAccessIntoScope(candidate, parent);
           handled = true;
@@ -643,7 +651,7 @@ std::vector<std::shared_ptr<AccessInfo>> RegisterizerAnalysis::getCandidates() {
 
 // RegisterizerReplacer
 
-const Expr* RegisterizerReplacer::mutate(const Load* v) {
+ExprPtr RegisterizerReplacer::mutate(const LoadPtr& v) {
   auto it = loadToAccess_.find(v);
   if (it == loadToAccess_.end()) {
     // This access cannot be registerized.
@@ -655,9 +663,9 @@ const Expr* RegisterizerReplacer::mutate(const Load* v) {
   return info->replacement().var;
 }
 
-Stmt* RegisterizerReplacer::mutate(const Store* v) {
+StmtPtr RegisterizerReplacer::mutate(const StorePtr& v) {
   if (eliminatedIntializers_.count(v) != 0) {
-    // This store is the intializer for a scalar var that is already inserted.
+    // This store is the initializer for a scalar var that is already inserted.
     return nullptr;
   }
 
@@ -669,22 +677,25 @@ Stmt* RegisterizerReplacer::mutate(const Store* v) {
 
   auto& info = it->second;
 
-  const Expr* new_val = v->value()->accept_mutator(this);
+  ExprPtr new_val = v->value()->accept_mutator(this);
 
-  return new Store(info->replacement().var_wrapper, {}, new_val, v->mask());
+  v->set_value(new_val);
+  v->set_buf(info->replacement().var_wrapper);
+  v->set_indices({});
+  return v;
 }
 
-Stmt* RegisterizerReplacer::mutate(const Block* v) {
+StmtPtr RegisterizerReplacer::mutate(const BlockPtr& v) {
   auto& scope = parentToAccesses_[v];
 
-  std::vector<Stmt*> stmts;
-  for (Stmt* stmt : v->stmts()) {
+  std::vector<StmtPtr> stmts;
+  for (const StmtPtr& stmt : v->stmts()) {
     {
       // Insert the initializer for any Scalars scoped to this block.
       auto it = scope.initializerPoints_.find(stmt);
       if (it != scope.initializerPoints_.end()) {
         for (auto& info : it->second) {
-          Stmt* initializer =
+          StmtPtr initializer =
               info->replacement().initializer->accept_mutator(this);
           stmts.push_back(initializer);
         }
@@ -692,7 +703,7 @@ Stmt* RegisterizerReplacer::mutate(const Block* v) {
       }
     }
 
-    Stmt* stmt_new = stmt->accept_mutator(this);
+    StmtPtr stmt_new = stmt->accept_mutator(this);
     if (stmt_new) {
       if (stmt_new->get_parent()) {
         stmt_new = Stmt::clone(stmt_new);
@@ -705,11 +716,8 @@ Stmt* RegisterizerReplacer::mutate(const Block* v) {
       auto it = scope.finalizePoints_.find(stmt);
       if (it != scope.finalizePoints_.end()) {
         for (auto& info : it->second) {
-          Store* finalizer = new Store(
-              info->buf(),
-              info->indices(),
-              info->replacement().var,
-              new IntImm(1));
+          StorePtr finalizer = alloc<Store>(
+              info->buf(), info->indices(), info->replacement().var);
           stmts.push_back(finalizer);
         }
         scope.finalizePoints_.erase(it);
@@ -717,27 +725,28 @@ Stmt* RegisterizerReplacer::mutate(const Block* v) {
     }
   }
 
-  return new Block(stmts);
+  return alloc<Block>(stmts);
 }
 
 void RegisterizerReplacer::buildReplacements() {
   // Traverse the list of replacements, creating vars and updating our local
   // maps.
   for (auto& info : infoSet_) {
-    Var* v = new Var(
+    VarPtr v = alloc<Var>(
         info->buf()->name_hint() + "_" +
-            c10::to_string(getBufferAccessCount(info->buf())),
+            std::to_string(getBufferAccessCount(info->buf())),
         info->buf()->dtype());
 
     info->replacement().var = v;
 
     // we need to wrap the Var in a Buf so we can Load or Store it.
-    info->replacement().var_wrapper = new Buf(v, {}, info->buf()->dtype());
+    info->replacement().var_wrapper =
+        alloc<Buf>(v, std::vector<ExprPtr>({}), info->buf()->dtype());
 
     bool first = true;
-    for (auto* s : info->stores()) {
+    for (const auto& s : info->stores()) {
       if (first && info->first_usage() == s && !info->firstUsageOverlapped()) {
-        info->replacement().initializer = new Let(v, s->value());
+        info->replacement().initializer = alloc<Let>(v, s->value());
         eliminatedIntializers_.insert(s);
       } else {
         storeToAccess_[s] = info;
@@ -746,7 +755,7 @@ void RegisterizerReplacer::buildReplacements() {
       first = false;
     }
 
-    for (auto* s : info->loads()) {
+    for (const auto& s : info->loads()) {
       loadToAccess_[s] = info;
     }
 
@@ -761,13 +770,8 @@ void RegisterizerReplacer::buildReplacements() {
 
     // create a default initializer by reading the access.
     if (info->replacement().initializer == nullptr) {
-      info->replacement().initializer = new Let(
-          v,
-          new Load(
-              info->buf()->dtype(),
-              info->buf(),
-              info->indices(),
-              new IntImm(1)));
+      info->replacement().initializer = alloc<Let>(
+          v, alloc<Load>(info->buf()->dtype(), info->buf(), info->indices()));
     }
   }
 }
@@ -775,13 +779,13 @@ void RegisterizerReplacer::buildReplacements() {
 } // namespace registerizer
 
 // Apply scalar replacement to all accesses in s.
-Stmt* registerize(Stmt* s) {
+StmtPtr registerize(StmtPtr s) {
   s = IRSimplifier::simplify(s);
 
   // The outermost node must be a Block so we have somewhere to put outer scope
   // scalars.
-  if (!dynamic_cast<Block*>(s)) {
-    s = new Block({s});
+  if (!to<Block>(s)) {
+    s = alloc<Block>(std::vector<StmtPtr>({s}));
   }
   registerizer::RegisterizerAnalysis analysis;
   s->accept(&analysis);
@@ -792,6 +796,4 @@ Stmt* registerize(Stmt* s) {
   return s;
 }
 
-} // namespace tensorexpr
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit::tensorexpr
