@@ -8,9 +8,10 @@ enum class OpKernelKind : uint8_t {
   kPrimKernel,
   kStaticDispatchKernel,
   kInterpreterFallbackKernel,
-  // static dispatch kernels that don't re-use
+  // static dispatch kernels that don't reuse
   // out TensorImpl
   kNativeStaticDispatchKernel,
+  kTritonKernel,
 };
 
 } // namespace torch::nativert
